#!/usr/bin/perl -w
use strict;
use IO;

my $root_dir = `pwd`; chomp($root_dir);
use vars qw($version); $version = '1.1.0';

### Begin Prefs ----------------------------------------------------------
use vars qw($prefs_dirty); $prefs_dirty = 0;

# Misc
use vars qw($pref_help); 
use vars qw($pref_verbose); 
use vars qw($pref_version); 
use vars qw($pref_safe);
use vars qw($pref_baloons);
use vars qw($pref_auto_hilight);
use vars qw($pref_memorized_txns);
use vars qw($pref_date_format);
use vars qw($pref_save_interval);
use vars qw($pref_auto_save);
use vars qw($pref_ledger_win_size);
use vars qw($pref_acct_win_height);
use vars qw($pref_start_date);
   
# Paths
use vars qw($pref_config_dir); 
use vars qw($pref_src_dir); 
use vars qw($pref_doc_dir); 
use vars qw($pref_image_dir); 
use vars qw($pref_tmp_dir); 

# Cryptography
use vars qw($pref_crypt);
use vars qw($pref_encrypt_cmd);
use vars qw($pref_decrypt_cmd);

# Fonts
use vars qw($pref_msg_text_font);
use vars qw($pref_button_font);
use vars qw($pref_fixed_header_font); 
use vars qw($pref_fixed_font);
use vars qw($pref_status_line_font);
use vars qw($pref_menu_font);
use vars qw($pref_dialog_font);
use vars qw($pref_default_font);
   
# Colors       
use vars qw($pref_heading_color);
use vars qw($pref_highlight_color);
use vars qw($pref_list_line1_color);
use vars qw($pref_list_line2_color); 
   
use vars qw($pref_web_browser);

# Diagnostics
use vars qw($pref_debug);
use vars qw($pref_devel);

my @args_info = 
  (  
   'General',
   ['help', 'boolean', \$pref_help, 0,
    'Provide application invocation information'], 
   ['verbose', 'boolean', \$pref_verbose, 0,
    'Spew lots of information about activities'], 
   ['version', 'boolean', \$pref_version, 0,
    'Output version information'], 
   ['safe', 'boolean', \$pref_safe, 0,
    'Don\'t allow anything unsafe (saving, exec, etc.)'], 
   ['balloons', 'boolean', \$pref_baloons, 1,
    'Enable balloon help'], # baloon,on
   ['auto-highlight', 'boolean', \$pref_auto_hilight, 1,
    'Auto highlight entry fields'], #auto_hilite
   ['memorized-txns', 'boolean', \$pref_memorized_txns, 1,
    'Use memorized transactions'], # use_mems
   ['date-format', 'string', \$pref_date_format, 'usa', 
    'Date format (usa or international)'], #date_fmt
   ['save-interval', 'integer', \$pref_save_interval, 180000,
    'Auto save interval (milliseconds)'], #save_interval
   ['auto-save', 'boolean', \$pref_auto_save, 0,
    'Auto save when switching files or quitting'],
   ['ledger-win-size', 'string', \$pref_ledger_win_size, '70x28',
    'Ledger window dimensions WxH'], #list_width/height
   ['account-win-height', 'integer',  \$pref_acct_win_height, 5,
    'Account window height'], #acctlist_width/height
   ['start-date', 'string', \$pref_start_date, '',
    'Start date (if defined, use the start date given)'], #use_sdate sdate
   
   'Paths',
   ['config-dir', 'string', \$pref_config_dir, $root_dir . '/etc',
    'Location of the configuration directory'], 
   ['src-dir', 'string', \$pref_src_dir, $root_dir . '/src',
    'Location of the program source'], 
   ['doc-dir', 'string', \$pref_doc_dir, $root_dir . '/doc',
    'Location of the documentation'], 
   ['image-dir', 'string', \$pref_image_dir, $root_dir . '/images',
    'Location of the image directory'], 
   ['tmp-dir', 'string', \$pref_tmp_dir, $root_dir . '/tmp',
    'Location of a directory for temporary files'], 

   'Cryptography',
   ['crypt', 'boolean', \$pref_crypt, 0,
    'Use cryptography'], #use_crypt
   ['encrypt-cmd', 'string', \$pref_encrypt_cmd, '',
    'Encryption command'], #encrypt
   ['decrypt-cmd', 'string', \$pref_decrypt_cmd, '',
    'Decryption command'], #decrypt
   
   'Fonts',
   ['msg-text-font', 'string', \$pref_msg_text_font,
    '-Adobe-Helvetica-Bold-R-Normal-*-*-120-*-*-*-*-*-*',
    'Message text font'],
   ['button-font', 'string', \$pref_button_font,
    '-Adobe-Helvetica-Bold-R-Normal-*-*-120-*-*-*-*-*-*',
    'Button font'],
   ['fixed-header-font', 'string', \$pref_fixed_header_font, '8x13',
    'Fixed header font'], 
   ['fixed-font', 'string', \$pref_fixed_font, '8x13',
    'Fixed font'],
   ['status-line-font', 'string', \$pref_status_line_font,
    '-Adobe-Helvetica-Bold-R-Normal-*-*-120-*-*-*-*-*-*',
    'Status line font'],
   ['menu-font', 'string', \$pref_menu_font,
    '-Adobe-Helvetica-Medium-R-Normal-*-*-120-*-*-*-*-*-*',
    'Menu font'],
   ['dialog-font', 'string', \$pref_dialog_font,
    '-Adobe-Helvetica-Bold-R-Normal-*-*-120-*-*-*-*-*-*',
    'Dialog font'],
   ['default-font', 'string', \$pref_default_font,
    '-Adobe-Helvetica-Medium-R-Normal-*-*-120-*-*-*-*-*-*',
    'Default font'],
   
   'Colors',       
   ['heading-color', 'string', \$pref_heading_color, '#00008f',
    'Heading color'], #head_color
   ['highlight-color', 'string', \$pref_highlight_color, '#8080b0',
    'Highlight color'], #hilite_color
   ['list-line1-color', 'string', \$pref_list_line1_color, 'white',
    'List line 1 color'],
   ['list-line2-color', 'string', $pref_list_line2_color, 'grey78',
    'List line 2 color'], 
   
   '',
   ['web-browser', 'string', \$pref_web_browser, 'netscape',
    'External web browser'],
   
   'Diagnostics',
   ['debug', 'boolean', \$pref_debug, 0,
    'Enable debugging'], 
   ['devel', 'boolean', \$pref_devel, 0,
    'Development mode']

   );

sub init_pref_defaults {
  my $arginfo;
  foreach $arginfo (@args_info) {
    if(ref($arginfo) eq 'ARRRAY') {
      $$$arginfo[2] = $$arginfo[3];
    }
  }
}

sub output_rc_boolean_arg {
  my($fh, $arg_info) = @_;
  my $name = $$arg_info[0];
  my $value = $$arg_info[3];
  my $description = $$arg_info[4];
  print $fh "# $description: boolean\n";
  if($value) {
    print $fh "--${name}\n\n";
  } else {
    print $fh "--no${name}\n\n";
  }
}

sub output_rc_string_arg {
  my($fh, $arg_info) = @_;
  my $name = $$arg_info[0];
  my $value = $$arg_info[3];
  my $description = $$arg_info[4];
  print $fh "# $description: string\n";
  if(defined($value) && $value) {
    print $fh "--${name}=${value}\n\n";
  } else {
    print $fh "#--$name=\n\n";
  }
}

sub output_rc_integer_arg {
  my($fh, $arg_info) = @_;
  my $name = $$arg_info[0];
  my $value = $$arg_info[3];
  my $description = $$arg_info[4];
  print $fh "# $description: integer\n";
  if($value) {
    print $fh "--${name}=${value}\n\n";
  } else {
    print $fh "--${name}=0\n\n";
  }
}

# write out current config file
sub save_config_file {
  my($filename) = @_;
  
  if($pref_safe) {
    return;
  }
  
  my $now_today = `date`;
  chomp $now_today;
  
  my $rchandle = new IO::File;
  
  if(!$rchandle->open(">$filename")) {
    ok_window("Cannot write to config file \`\`$filename\'\'.  " .
              "Current settings will not be saved.");
  } else {
    print $rchandle "# CBB config file: $filename\n#\n";
    
    print $rchandle "# Created by CBB $version on $now_today\n#\n";
    
    print $rchandle "# This is machine generated code.  Please use caution\n";
    print $rchandle "# when hand editing!\n#\n";

    print $rchandle "# There are three types of arguments.\n";
    print $rchandle "#     boolean: --arg or --noarg\n";
    print $rchandle "#      string: --arg=value\n";
    print $rchandle "#     integer: --arg=value\n";
    
    my $arg;
    foreach $arg (@args_info) {
      if(ref($arg) ne 'ARRAY') {
        print $rchandle "\n### $arg\n\n";
      } elsif($$arg[1] eq 'boolean') {
        output_rc_boolean_arg($rchandle, $arg);
      } elsif($$arg[1] eq 'string') {
        output_rc_string_arg($rchandle, $arg);
      } elsif($$arg[1] eq 'integer') {
        output_rc_integer_arg($rchandle, $arg);
      } else {
        print STDERR
          "Unrecognized arg --$$arg[0] during config file write.\n";
      }
    }    
    $rchandle->close();
    $prefs_dirty = 0;
  }
}

init_pref_defaults();
save_config_file('foobar');


