#!/usr/bin/perl -w
use English;
use strict;
use Tk;
use IO;

# A balance win is represented by a hash containing:
#  'acct' -- the relevant account

# To do

# Handle adding/deleting transactions

# Center on item on double click.

#------------------------------------------------------------------------------
# Procedures for balancing
#------------------------------------------------------------------------------

sub balance_win_txn_to_text {
  my($acct, $txn) = @_;
  my $source = $txn->get_source();
  my($total_debit, $total_credit) = $txn->totals_wrt($acct);
  my $result;

  return(sprintf("%5.5s  %-8.8s  %-29.29s  %9.2f  %9.2f  %-1.1s",
                 $txn->get_checkno(),
                 $txn->get_date(),
                 $txn->get_desc(),
                 $total_debit, $total_credit,
                 $txn->get_status_wrt($acct)),
         $total_debit, $total_credit);
}


sub balance_win_double_clicked {
  my($list, $acct, $txns) = @_;
  my @selection = $list->curselection();

  debug('balance_win: ' . join(" ", @selection) . "\n");

  my $index;
  foreach $index (@selection) {
    my $txn = $$txns[$index];
    if($txn->clear_pending_wrt_p($acct)) {
      debug("balance_win: unclear pending $txn\n");
      $txn->unclear_wrt($acct);
    } else {
      debug("balance_win: clear_pending $txn\n");
      $txn->clear_pending_wrt($acct);
    }
  }
}

sub balance_win_update {
  my($acct, $txns, $txn_display, 
     $start_balance_ref, $end_balance_ref, $totals_label) = @_;
  
 main::debug("balance_win_update: " . join(" ", @_) . "\n");

  $$start_balance_ref = 0;
  my $debits = 0;
  my $credits = 0;
  my $ledger = $acct->acquire_ledger();
  splice @$txns, 0;
  my @relevant_transactions = map {
    my $txn = $$_[0];
    if($txn->uncleared_wrt_p($acct) || $txn->clear_pending_wrt_p($acct)) {
      push @$txns, $txn;
      my($text, $debit, $credit) = balance_win_txn_to_text($acct, $txn);
      if($txn->clear_pending_wrt_p($acct)) {
        $debits += $debit;
        $credits += $credit;
      }
      $text;
    } else {
      if($txn->cleared_wrt_p($acct)) {
        my($total_debit, $total_credit) = $txn->totals_wrt($acct);
        $$start_balance_ref += ($total_credit - $total_debit);
      }
      ();
    }
  } @$ledger;
  $acct->release_ledger();
  undef $ledger;

  $txn_display->delete(0, 'end');
  $txn_display->insert('end', @relevant_transactions);
  $txn_display->configure(-width => 0);

  my $diffs = $$start_balance_ref - $$end_balance_ref - $debits + $credits;
  $totals_label->configure(-text => 
                           sprintf("Debits = %.2f  Credits = %.2f  " . 
                           "Difference = %.2f", $debits, $credits, $diffs));
}

sub balance_win_txns_modified_callback {
  my($db, $modifications, $affected_sinks, $data) = @_;
  my($acct, $txns) = @$data;

  debug("balance_win: txns modified\n");
  my $affected_data = $$affected_sinks{$acct};

  map {
    print STDERR $$_[0]->get_name() . "\n";
  } values(%$affected_sinks);

  map {
    my $txn = $$_[0];
    my $mods = $$_[1];
    print STDERR "[ $txn\n  ";
    map {
      print STDERR "  [" . join(" ", @$_) . "]\n";
    } @$mods; 
      
      print STDERR "]\n";
  } @$modifications;

  
  if($affected_data) {
    my ($acct, $mod_txns, $indices, $moves) = @$affected_data;
    my $found = 0;
    
    my $i = 0;
    SEARCH:
    while(!$found && $i < scalar(@$mod_txns)) { 
      my $mod_txn = $$mod_txns[$i]; 
      my $j = 0;
      while(!$found && $j < scalar(@$txns)) { 
        if($$txns[$j] == $mod_txn) {
          $found = $$txns[$j];
          last SEARCH;
        }
        $j++;
      }
      $i++;
    }
    
    if($found) {
      debug("balance-win: found relevant modification $found, updating.\n");
      balance_win_update(@$data);
    } else {
      debug("balance-win: no relevant txns in modification event.\n");
    }
  }
}


sub balance_win_close {
  my($win, $modify_callback) = @_;
  
  if($$modify_callback) {
    $main::db->remove_modify_txn_callback($$modify_callback);
  }
}

sub balance_win_cancel {
  my($win) = @_;
  $win->destroy();
}

sub balance_win_ok {
  my($win, $acct, $txns, $start_balance_ref, $end_balance_ref) = @_;
  my $debits = 0;
  my $credits = 0;
  my $ledger = $acct->acquire_ledger();
  my @txns = ();
  map {
    my $txn = $$_[0];
    if($txn->clear_pending_wrt_p($acct)) {
      push @txns, $txn;
      my($debit, $credit) = $txn->totals_wrt($acct);
      $debits += $debit;
      $credits += $credit;
    }
  } @$ledger;
  $acct->release_ledger();
  undef $ledger;
  my $diffs = $$start_balance_ref - $$end_balance_ref - $debits + $credits;

  if(abs($diffs) > $zero_epsillon) {
    ok_window(sprintf("You have not reconciled the account.\n" . 
                      "There is still \$%.2f unaccounted for.",
                      $diffs));
  } else {
    $main::db->begin_txn_modifications();
    my $txn;
    foreach $txn (@txns) {
      print STDERR "Clearing txn\n";
      print STDERR "  " . $txn->get_status_wrt($acct) . "\n";
      $txn->clear_wrt($acct);
      $txn->print(\*STDERR, "  ");
    }
    $main::db->end_txn_modifications();
    $win->destroy();
  }
}

# create window
sub new_balance_win {
  my($acct) = @_;
  # Presume that window doesn't already exist.

  my $modify_callback_id;
  
  my $bal = $main::mw->Toplevel();
  $bal->OnDestroy(sub { balance_win_close($bal, \$modify_callback_id); });

  $bal->optionAdd('*font' => $pref_dialog_font);
  $bal->title('Balance: ' . $acct->get_name());
  $bal->iconname('Balance ' . $acct->get_name());

  my $start_balance = '0.00';
  my $end_balance = '0.00';
  my $debits = '0.00';
  my $credits = '0.00';
  my $diffs = '0.00';

  my $frame = $bal->Frame(-borderwidth => 2, -relief => 'raised');
  my $head1 = $frame->Frame(-relief => 'raised');
  my $head2 = $frame->Frame(-relief => 'raised');
  my $head3 = $frame->Frame(-relief => 'raised');
  my $f     = $frame->Frame(-relief => 'raised');
  my $bar   = $frame->Frame(-relief => 'sunken');
  
  my $h1_label = $head1->Label(-text => 'Statement Starting Balance = ');
  my $h1_entry = $head1->Entry(-textvariable => \$start_balance,
                               -relief => 'sunken',
                               -width => 17,
                               -font => $pref_default_font);
  my $h2_label = $head2->Label(-text => 'Statement Ending Balance = ');
  my $h2_entry = $head2->Entry(-textvariable => \$end_balance,
                               -relief => 'sunken',
                               -width => 17,
                               -font => $pref_default_font);
  my $balance_totals = 
    $head3->Label(-text => 
                  "Debits = $debits  Credits = $credits  Difference = $diffs");
  
  my $txn_display =
    $f->Listbox(-width => 47, -height => 15, -relief => 'sunken',
                -exportselection => 0, -takefocus => 0,
                -font => $pref_fixed_font);
  
  my $txns = [];
  balance_win_update($acct, $txns, $txn_display, 
                     \$start_balance, \$end_balance,
                     $balance_totals);
  

  $modify_callback_id = 
    $main::db->add_txns_modified_callback(\&balance_win_txns_modified_callback,
                                          [$acct, $txns, $txn_display, 
                                           \$start_balance, \$end_balance,
                                           $balance_totals]);
  

  $txn_display->bind('<Double-Button-1>', sub {
    my $list = shift;
    debug('balance-win: <Double-Button-1>' . "\n");
    balance_win_double_clicked($list, $acct, $txns);
  });

  $bal->bind('<KeyPress-Down>', sub { 
    $txn_display->yview('scroll' => 1, 'units'); });
  $bal->bind('<Control-KeyPress-n>', sub { 
    $txn_display->yview('scroll' => 1, 'units'); });
  $bal->bind('<KeyPress-Up>', sub { 
    $txn_display->yview('scroll' => -1, 'units'); });
  $bal->bind('<Control-KeyPress-p>', sub { 
    $txn_display->yview('scroll' => -1, 'units'); });
  $bal->bind('<KeyPress-Next>', sub { 
    $txn_display->yview('scroll' => 1, 'units'); });
  $bal->bind('<Control-KeyPress-v>', sub { 
    $txn_display->yview('scroll' => 1, 'units'); });
  $bal->bind('<KeyPress-Prior>', sub { 
    $txn_display->yview('scroll' => -1, 'units'); });
  $bal->bind('<Alt-KeyPress-v>', sub { 
    $txn_display->yview('scroll' => -1, 'units'); });

  my $f_scroll = $f->Scrollbar(-takefocus => 0, -relief => 'flat',
                               -command => [ $txn_display => 'yview' ]);
  $txn_display->configure(-yscrollcommand => [ $f_scroll => 'set' ]);

	
  my $bar_cancel = $bar->Button(-text => 'Cancel',
                                -font => $pref_button_font,
                                -takefocus => 0,
                                -command => sub {
                                  balance_win_cancel($bal);
                                });
  
  #my $bar_update = $bar->Button(-text => 'Update',
  #                              -font => $main::cbb{'button-font'},
  #                              -takefocus => 0,
  #                              -command => sub {
  #                                balanceUpdateSelected($txn_display);
  #                                clear_entry_area();
  #                                $bal->destroy();
  #                              });
  
  my $bar_dismiss = $bar->Button(-text => 'Ok', 
                                 -font => $pref_button_font,
                                 -takefocus => 0,
                                 -command => sub {
                                   balance_win_ok($bal, $acct, $txns,
                                                  \$start_balance,
                                                  \$end_balance);
                                 });

  $frame->pack(-side => 'top', -fill => 'both', -expand => 1);
  $head1->pack(-side => 'top', -fill => 'x');
  $head2->pack(-side => 'top', -fill => 'x');
  $head3->pack(-side => 'top', -fill => 'x');
  $bar->pack(-side => 'bottom', -fill => 'x');
  $f->pack(-side => 'top', -fill => 'both', -expand => 1);

  $h1_label->pack(-side => 'left', -anchor => 'w');
  $h1_entry->pack(-side => 'left', -anchor => 'w');
  $h2_label->pack(-side => 'left', -anchor => 'w');
  $h2_entry->pack(-side => 'left', -anchor => 'w');
  $balance_totals->pack(-side => 'top', -anchor => 'w');

  $f_scroll->pack(-side => 'right', -fill => 'y');
  $txn_display->pack(-side => 'left', -fill => 'both', -expand => 1);
  $bar_cancel->pack(-side => 'left', -padx => 8, -pady => 4);
  #$bar_update->pack(-side => 'left', -expand => 1, -fill => 'x',
  #                   -padx => 8, -pady => 4);
  $bar_dismiss->pack(-side => 'right', -padx => 8, -pady => 4);

#   $head1_entry->bind('<FocusOut>', sub {
#     $diff = balanceCalcDiff();
#     $head3_label->configure( -text => "Debits = $debits  " .
#                             "Credits = $credits  Difference = $diff");
#   });
  
#   $head2_entry->bind('<FocusOut>', sub {
#     $diff = balanceCalcDiff();
#     $head3_label->configure( -text => "Debits = $debits  " .
#                             "Credits = $credits  Difference = $diff");
#   });

#   $head2_entry->bind('<FocusOut>', sub {
#     $diff = balanceCalcDiff();
#     $head3_label->configure( -text => "Debits = $debits  " .
#                             "Credits = $credits  Difference = $diff");
#   });
  
#   $f_list->bind('<Double-Button>', sub {
#     acctSetDirty();
#     balanceUpdateList($f_list, $f_list->curselection());
#     $diff=balanceCalcDiff();
#     $head3_label->configure( -text => "Debits = $debits  " .
#                             "Credits = $credits  Difference = $diff");
#   });
  $h1_entry->focus();


}

1;
__END__

# ----------------------------------------------------------------------------
# $Log: balance-win.pl,v $
# Revision 1.4  1998/02/14 00:58:56  rlb
# minor copy paste glitch fixed
#
# Revision 1.3  1998/01/24 02:21:24  rlb
# Many changes.  Hopefully I'll be better about commits now.
#
# Revision 1.2  1997/10/22 03:45:59  rlb
# Working (before txn data stucture switch)
#
# Revision 1.1  1997/10/10 18:15:53  rlb
# Initial submission
#
# Revision 2.8  1997/04/03 03:40:00  curt
# Fixed a small balance bug introduced recently.
#
# Revision 2.7  1997/03/05 18:58:32  curt
# Added additional bindings to scrolling lists: categories list, balance
# window list, text help, and report list.
#
# Revision 2.6  1997/02/19 18:08:05  curt
# Fixed a bug with long check numbers (> 5 char) in the balance window.
# Nailed a bug which caused cbb to forget the last check # entered when
# using +/- in the check number field.
#
# Revision 2.5  1997/01/02 04:38:30  curt
# Changes over the 1996 holidays:
#   - Converted listbox to text widget.  This allows us to do nice
#     things with alternating background colors, highliting, red
#     negative numbers, etc.
#   - Negative transactions are now drawn in red.
#   - Added a Goto <Today> option.
#   - <Home> & <End> were double bound.  Now, listbox can be traversed with
#     <Meta-Home> and <Meta-End>
#
# Revision 2.4  1996/12/17 20:15:43  curt
# Version incremented to 0.70.
# No longer save running total in .cbb files.
# Miscellaneous tweaks.
#
# Revision 2.3  1996/12/17 14:53:51  curt
# Updated copyright date.
#
# Revision 2.2  1996/12/11 04:15:14  curt
# Added a "Refresh" button.  Reworked some of the code.
#
# Revision 2.1  1996/12/08 07:37:50  curt
# Initial revision.
#
