#!/usr/bin/perl -w
#
#  'CBB' -- Check Book Balancer
#
#   balloon.pl -- Routines for handling balloon help.
#
#  Written by Stewart Allen <stewart@neuron.com> as part of the
#    Visual Tcl application.
#
#  Modified by Curtis L. Olson to work with in the framework of CBB.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Id: balloon.pl,v 1.2 1998/01/24 02:21:24 rlb Exp $
# (Log is kept at end of this file)

use strict;

my $current_balloon;
my %balloon_message;

my $balloon_set;
my $balloon_first;

my $after_proc_id;

sub set_balloon {
  my($target, $message) = @_;
  $balloon_message{$target} = $message;
  my @tags = $target->bindtags();
  push @tags, 'cbb-balloon';
  $target->bindtags(\@tags);
}

sub kill_balloon {
  $after_proc_id->cancel();
  if($current_balloon && $current_balloon->Exists()) {
    $current_balloon->destroy();
  }
  $balloon_set = 0;
}

sub raise_balloon {
  my($target) = @_;
  my $message = $balloon_message{$target};
  
  if($balloon_first && $pref_balloons) {
    $balloon_first = 2;
    
    my $x = int($target->rootx() + ($target->width() / 2) + 0.5);
    my $y = int($target->rooty() + $target->height() + 4 + 0.5);

    $current_balloon = $target->Toplevel(-bg => 'black');
    $current_balloon->overrideredirect(1);

    my $label =
      $current_balloon->Label(-text => $message, -relief => 'flat',
                              -font => $pref_default_font,
                              -bg => '#ffffaa', -fg => 'black',
                              -padx => 2, -pady => 0, -anchor => 'w');
    $label->pack(-side => 'left', -padx => 1, -pady => 1);
    $current_balloon->geometry("+$x+$y");
    $balloon_set = 1;
  }
}

$mw->bind('cbb-balloon', '<Enter>', sub {
  my($widget) = @_;
  $balloon_set = 0;
  $balloon_first = 1;
  $after_proc_id = $widget->after(500, sub { raise_balloon($widget); });
});
                             
$mw->bind('cbb-balloon', '<Leave>', sub {
  $balloon_first = 0;
  kill_balloon();
});

$mw->bind('cbb-balloon', '<Motion>', sub {
  my($widget) = @_;
  if($balloon_set == 0) {
    $after_proc_id->cancel();
    $after_proc_id = $widget->after(500, sub { raise_balloon($widget); });
  }
});

1;
__END__

# ----------------------------------------------------------------------------
# $Log: balloon.pl,v $
# Revision 1.2  1998/01/24 02:21:24  rlb
# Many changes.  Hopefully I'll be better about commits now.
#
# Revision 1.1  1997/10/10 18:15:53  rlb
# Initial submission
#
# Revision 1.2  1996/12/11 04:17:38  curt
# Added cvs headers/log.
#
