#!/usr/local/bin/perl -w
#
#  'CBB' -- Check Book Balancer
#
#   config.pl -- Code to handle state and preferences
#
#  Copyright (C) 1994 - 1997  Curtis L. Olson  - curt@sledge.mn.org
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Id: config.pl,v 1.1 1998/01/24 02:29:48 rlb Exp $
# (Log is kept at end of this file)


sub save_state_file {
  my($filename) = @_;
  # The state file should eventually keep track of open windows and window
  # positions on quit so the envt can be optionally restored next session.

  if($pref_safe) {
    return;
  }
  
  my $now_today = get_sys_date();

  my $rchandle = new IO::File;
  
  if(!$rchandle->open(">$filename")) {
    ok_window("Cannot write to \`\`$filename\'\'.  " .
              "Current state will not be saved.");
  } else {
    print $rchandle "# CBB state file: $filename\n#\n";

    print $rchandle "# Created by CBB $version on $now_today.\n#\n";

    print $rchandle "# This is machine generated code.  Please use caution\n";
    print $rchandle "# when hand editing!\n\n";

    print $rchandle "# File to load when CBB initially starts.\n";
    print $rchandle 'current-database: ' . db_get_name . "\n\n";
    
    $rchandle->close();
  }
}


sub get_state_file_value {
  my($key, $contents) = @_;
  # Get the final value from the file

  my @candidates = grep { /\A\s*$key:\s*(.*?)\s*\Z/o } @$contents;
  
  if(scalar(@candidates) > 1) {
    printf STDERR
      "cbb: warning -- multiple values in state file for key $key\n";
  }

  if(scalar(@candidates)) {
    return $candidates[$#candidates];
  } else {
    return undef;
  }
}

sub load_state_file {
  my($filename) = @_;

  my $rchandle = new IO::File;
  if(!$rchandle->open("<$filename")) {
    ok_window("Cannot read from state file \`\`$filename\'\'.  " .
              "Previous state will not be restored.");
  } else {
    my @contents = <$rchandle>;
    
    @contents = grep { !/\A\s*\#.*\Z/o; } @contents; # Ignore comments

    my $tmp = get_state_file_value('current-database', \@contents);
    if(defined($tmp)) { $main::cbb{'cur-file'} = $tmp; } 

    $rchandle->close();
  }  
}


sub output_rc_boolean_arg {
  my($fh, $arg_info) = @_;
  my $name = $$arg_info[0];
  my $value = $$arg_info[3];
  my $description = $$arg_info[4];
  print $fh "# $description: boolean\n";
  if($value) {
    print $fh "--${name}\n\n";
  } else {
    print $fh "--no${name}\n\n";
  }
}

sub output_rc_string_arg {
  my($fh, $arg_info) = @_;
  my $name = $$arg_info[0];
  my $value = $$arg_info[3];
  my $description = $$arg_info[4];
  print $fh "# $description: string\n";
  if(defined($value) && $value) {
    print $fh "--${name}=${value}\n\n";
  } else {
    print $fh "#--$name=\n\n";
  }
}

sub output_rc_integer_arg {
  my($fh, $arg_info) = @_;
  my $name = $$arg_info[0];
  my $value = $$arg_info[3];
  my $description = $$arg_info[4];
  print $fh "# $description: integer\n";
  if($value) {
    print $fh "--${name}=${value}\n\n";
  } else {
    print $fh "--${name}=0\n\n";
  }
}

# write out current config file
sub save_config_file {
  my($filename) = @_;
  
  if($pref_safe) {
    return;
  }
  
  my $now_today = `date`;
  chomp $now_today;
  
  my $rchandle = new IO::File;
  
  if(!$rchandle->open(">$filename")) {
    ok_window("Cannot write to config file \`\`$filename\'\'.  " .
              "Current settings will not be saved.");
  } else {
    print $rchandle "# CBB config file: $filename\n#\n";
    
    print $rchandle "# Created by CBB $version on $now_today\n#\n";
    
    print $rchandle "# This is machine generated code.  Please use caution\n";
    print $rchandle "# when hand editing!\n#\n";

    print $rchandle "# There are three types of arguments.\n";
    print $rchandle "#     boolean: --arg or --noarg\n";
    print $rchandle "#      string: --arg=value\n";
    print $rchandle "#     integer: --arg=value\n";
    
    my $arg;
    foreach $arg (@args_info) {
      if(ref($arg) ne 'ARRAY') {
        print $rchandle "\n### $arg\n\n";
      } elsif($$arg[1] eq 'boolean') {
        output_rc_boolean_arg($rchandle, $arg);
      } elsif($$arg[1] eq 'string') {
        output_rc_string_arg($rchandle, $arg);
      } elsif($$arg[1] eq 'integer') {
        output_rc_integer_arg($rchandle, $arg);
      } else {
        print STDERR
          "Unrecognized arg --$$arg[0] during config file write.\n";
      }
    }    
    $rchandle->close();
    $prefs_dirty = 0;
  }
}

1;
__END__

# $Log: config.pl,v $
# Revision 1.1  1998/01/24 02:29:48  rlb
# Initial CVS submission.
#
