#!/usr/bin/perl -w
#
#  'CBB' -- Check Book Balancer
#
#   db-win.pl -- database main window routines.
#
#  Written by Curtis Olson.  Started August 25, 1994.
#
#  Copyright (C) 1994 - 1997  Curtis L. Olson  - curt@sledge.mn.org
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Id: db-win.pl,v 1.2 1998/02/14 01:02:21 rlb Exp $
# (Log is kept at end of this file)

use strict;
use English;
use Tk;
use Tk::ROText;

my $menubar = '';
my $status_line = '';

my %db_win_menu_items;

# Widget to hold the list of accounts in the main win.
my $accts_frame = '';
my $accts_list = '';

my $elapsed_offset = 0;
sub elapsed_reset {
  my $prefix = shift;
  my($user,$system) = times;
  $elapsed_offset = $user + $system;
  print STDERR $prefix . " ... " if $prefix;
}
sub elapsed {
  my $prefix = shift;
  print STDERR $prefix if $prefix;
  my($user,$system) = times;
  print STDERR '(elapsed time: ' . ($user + $system - $elapsed_offset) . ")\n";
}


my $currently_selected_acct;

sub set_currently_selected_acct {
  my $acct = shift;
  $currently_selected_acct = $acct;
  
  my $data = $db_win_menu_items{"Edit\tOpen Ledger"};

  if(!$acct) {
    $$data[0]->entryconfigure($$data[1], -state => 'disabled');
  } else {
    $$data[0]->entryconfigure($$data[1], -state => 'normal');
  }
}

sub get_currently_selected_acct {
  return $currently_selected_acct;
}

sub open_ledger_for_current_selection {
  open_or_raise_ledger_win(get_currently_selected_acct());  
}

sub set_acct_list_contents {
  my($db) = @_;
  my $acct;
  my $accts = $db->get_accts();
  $accts_list->delete('0.0', 'end');
  my $num_accts = 0;
  foreach $acct (@$accts) {
    if($acct) {
      # Doesn't matter what the tag is as long as it's unique.
      my $tag = "$acct";
      my $notes = $acct->get_notes();
      $notes = "" if ! $notes;
      my $final_balance = $acct->get_final_balance();
      $accts_list->insert('end', sprintf("%-16s %-37s %11.2f\n",
                                         $acct->get_name(),
                                         $notes,
                                         $final_balance), $tag);

      $accts_list->tagBind($tag, '<Double-Button>', sub {
        set_currently_selected_acct($acct);
        open_ledger_for_current_selection();
      });
      $accts_list->tagBind($tag, '<Button>', sub {
        set_currently_selected_acct($acct);
      });
      $num_accts++;
    }
  }
  $accts_list->configure(-height => $num_accts);
  elapsed();
}

#------------------------------------------------------------------------------
# Setup container frames
#------------------------------------------------------------------------------

sub setup_containers {
  my($mw) = @_;
  my $menubar = $mw->Frame(-relief => 'raised', -borderwidth => 2);
  my $head = $mw->Frame(-relief => 'raised',  -borderwidth => 2);
  my $trans = $mw->Frame(-relief => 'raised', -borderwidth => 2);
  my $entry = $mw->Frame(-relief => 'raised', -borderwidth => 2);
  my $bar = $mw->Frame(-borderwidth => 2);
  my $acct = $mw->Frame(-relief => 'raised', -borderwidth => 2);
  my $status = $mw->Frame(-relief => 'raised', -borderwidth => 2);
  
  $menubar->pack(-fill => 'x'); 
  $head->pack(-fill => 'x'); 
  $trans->pack(-fill => 'both', -expand => 1); 
  $entry->pack(-fill => 'both'); 
  $bar->pack(-fill => 'x'); 
  $acct->pack(-fill => 'x'); 
  $status->pack(-fill => 'x'); 
  
  return($menubar, $head, $trans, $entry, $bar, $acct, $status);
}

#------------------------------------------------------------------------------
# Setup the command bar
#------------------------------------------------------------------------------

sub setup_command_bar {
  my($parent) = @_;
  
  $parent->optionAdd('*font' => $pref_button_font);
  
  my $new =
    $parent->Button(-text => 'New', -takefocus => 0,
                    -command => \&clear_entry_area);
  cbb_set_balloon($new,
                  'Cancel changes and start a new transaction <Meta-N>');
  
  my $edit = 
    $parent->Button(-text => 'Edit', -takefocus => 0,
                    -command => sub {
                      if(listGetCurTrans() >= 1) {
                        update_entry_area(listGetCurTrans() .0);
                      }
                    });
  cbb_set_balloon($edit, 'Edit the selected transaction <Meta-E>');
  
  my $delete = 
    $parent->Button(-text => 'Delete', -takefocus => 0,
                    -command => sub {
                      if(listGetCurTrans() >= 1) {
                        if($pref_debug) {
                          print 'Delete ' . listGetCurTrans() .0;
                        }
                        delete_trans(listGetCurTrans() .0);
                      }
                    });
  cbb_set_balloon($delete, 'Delete the selected transaction');

  my $splits = 
    $parent->Button(-text => 'Splits', -takefocus => 0,
                    -command => sub { 
                      my $win = cbbWindow_splits();
                      $win->waitWindow();
                    });
  
  cbb_set_balloon($splits, 'Open the splits window <Meta-S>');

  my $balance = 
    $parent->Button(-text => 'Balance', -takefocus => 0,
                    -command => \&balance);
  cbb_set_balloon($balance, 'Open the balance window');
  
  my $save =
      $parent->Button(-text => 'Save', -takefocus => 0, -command => \&db_save);
  cbb_set_balloon($save, 'Save the current file');
  
  my $quit = 
      $parent->Button(-text => 'Quit', -takefocus => 0, -command => \&cbbQuit);
  cbb_set_balloon($quit, 'Quit <Meta-Q>');
  
  my $b;
  foreach $b ($new, $edit, $delete, $splits, $balance, $save, $quit) {
    $b->pack(-side => 'left', -fill => 'x',
             -expand => 1, -padx => 2, -pady => 1);
  }
}

#------------------------------------------------------------------------------
# Setup the status line
#------------------------------------------------------------------------------

sub setup_status_line {
  my($parent) = @_;

  $status_line = 
    $parent->Label(-text => "Welcome to $0", 
                   -font => $pref_status_line_font);
  $status_line->pack(-fill => 'x');
}

sub status_message {
  my($message) = @_;
  $status_line->configure(-text => $message);
}

#------------------------------------------------------------------------------
# Setup main window
#------------------------------------------------------------------------------

sub acct_win_reset_title {
  my($mw) = @_;

  my $title = "$0";
  if(db_get_name()) {
    $title .= " - " . db_get_name();
  } else {
    $title .= " - <<untitled>>";
  } 
  $mw->title($title);
  $mw->iconname($title);
}  

sub setup_main {
  my($mw) = @_;
  
  # Setup main window parameters
  acct_win_reset_title($mw);

  $mw->iconbitmap("@" . $pref_image_dir . '/'. $icon_image);
  # specify absolute placement
  $mw->geometry('+0+0');
  # The following options will enable window resizing
  $mw->minsize(100, 50);
  $mw->maxsize(1000, 700);

  $mw->optionAdd('*font' => $pref_default_font);

  $menubar = $mw->Frame(-relief => 'raised', -borderwidth => 2);
  $menubar->pack(-fill => 'x'); 

  
  
  $accts_frame = $mw->Frame(-relief => 'flat', -borderwidth => 3);
  
  my $titles = 
    $accts_frame->Label(-font => $pref_fixed_header_font,
                        -text => 
                        sprintf('%-16s %-37s %11s',
                                'Name', 'Notes', 'Total'),
                        -padx => 4,
                        -foreground => $pref_heading_color);
  $titles->pack(-side => 'top', -anchor => 'w');


  $accts_list = $accts_frame->ROText(-relief => 'sunken', -borderwidth => 2,
                                     -height => 20,
                                     -width => 66,
                                     -font => $pref_fixed_font);

  $accts_list->pack(-fill => 'both', -expand => 1);

  $accts_frame->pack(-side => 'top', -fill => 'both', -expand => 1);

  setup_status_line($mw);
  
  # Setup menu bar
  $mw->optionAdd('*font' => $pref_menu_font);

  my %menu_buttons = setup_main_menubar($menubar);

  my $file_menu = setup_main_file_menu($menu_buttons{'file'}, '');
  my $edit_menu = setup_main_edit_menu($menu_buttons{'edit'}, '');
  my $prefs_menu = setup_main_prefs_menu($menu_buttons{'prefs'}, '');

  #setup_functions_menu .menubar
  #setup_extern_menu .menubar
  
  my $devel_menu = '';
  if($menu_buttons{'devel'}) {
    $devel_menu = setup_main_devel_menu($menu_buttons{'devel'}, $devel_menu);
  }

  my $help_menu = menubutton_add_help_menu($menu_buttons{'help'});

  if($pref_auto_highlight) {
    setup_auto_hilite();
  }
  
  $mw->update();
  $mw->deiconify();

  if($cmd_line_load_file) {
    db_load($cmd_line_load_file);
    $cmd_line_load_file = undef;
  }
}

sub setup_prefs_appear_menu {
  my($parent) = @_;

  my $menu = $parent->Menu(-tearoff => 0);
  
  $menu->add('checkbutton', -label => 'Use Baloon Help',
             -variable => \$pref_balloons,
             -command => sub { $prefs_dirty = 1; });
  
  $menu->add('checkbutton', -label => 'Use Auto Highlighting',
             -variable => \$pref_auto_highlight,
             -command => sub {
               $prefs_dirty = 1;
               if($pref_auto_highlight) {
                 setup_auto_hilite();
               } else {
                 #???
                 #bind Entry <FocusIn> {}
                 #bind Entry <FocusOut> {}
               }
             });
  
  $menu->add('checkbutton', -label => 'Use US Date Format',
             -variable => \$pref_date_format,
             -onvalue => 'usa', -offvalue => 'international',
             -command => sub { $prefs_dirty = 1; });
  
  $menu->add('separator');
  
  $menu->add('command', -label => 'Set Dimensions',
             -command => \&setDimensions);
  
  return $menu;
}

sub setup_prefs_crypt_menu {
  my($parent) = @_;
  
  my $menu = $parent->Menu(-tearoff => 0);

  $menu->add('checkbutton', -label => 'Use Cryptography',
             -variable => \$pref_crypt,
             -command => \&useCrypt);

  $menu->add('separator');

  $menu->add('command', -label => 'Set Cryptcode...',
             -command => \&setCryptCode);
  $menu->add('command', -label => 'Set Cryptprogram...',
             -command => \&setCryptProgram);

  return $menu;
}

sub setup_main_prefs_menu {
  my($menubutton, $menu) = @_;  

  if($menu && $menu->Exists()) { $menu->destroy() }
  $menu = $menubutton->Menu(-tearoff => 0);
  
  $menu->add('checkbutton', -label => 'Use Memorized Transactions',
             -variable => \$pref_memorized_txns,
             -command => sub { $prefs_dirty = 1; });
  
  $menu->add('separator');
  
  $menu->add('checkbutton', -label => 'Use Startdate',
             -variable => \$pref_start_date,
             -command => sub { $prefs_dirty = 1; });

  $menu->add('command', -label => 'Set Startdate ...',
             -command => \&setStartDate);

  $menu->add('separator');
  
  $menu->add('cascade', -label => 'Appearence',
             -menu => setup_prefs_appear_menu($menu));
  
  $menu->add('cascade', -label => 'Cryptography',
             -menu => setup_prefs_crypt_menu($menu)); 
  
  $menu->add('separator');
  
  $menu->add('command', -label => 'Save Preferences',
             -command => \&cbbrcSave);

  $menubutton->configure(-menu => $menu);
  return $menu;
}

sub setup_main_devel_menu {
  my($menubutton, $devel_menu) = @_;

  if($devel_menu && $devel_menu->Exists()) { $devel_menu->destroy() }

  $devel_menu = $menubutton->Menu(-tearoff => 0);
  
  my $file;
  foreach $file (@main::source_files) {
    $file =~ m|.*?([^/]+)$|mo;
    my $basename = $1;
    $devel_menu->add('command', -label => "Re-source '$1'",
                     -command => sub {
                       my $result = do $file;
                       die $@ if $@;
                       die "$file did not return true value" unless $result;
                     });
  }
  $menubutton->configure(-menu => $devel_menu);
  return $devel_menu;
}

sub setup_main_edit_menu {
  my($menubutton, $menu) = @_;  
  
  if($menu && $menu->Exists()) { $menu->destroy() }
  $menu = $menubutton->Menu(-tearoff => 0);

  my $item_num = 0;

  $menu->add('command', -label => 'Undo',
             -underline => 0,
             -command => \&db_win_undo,
             -state => 'disabled');
  $item_num++;
  
  $menu->add('separator');
  $item_num++;

  $menu->add('command', -label => 'New Account...',
             -underline => 0,
             -command => \&unimplemented_window);  
  $item_num++;

  $menu->add('command', -label => 'Edit Account...',
             -underline => 0,
             -command => \&unimplemented_window);  
  $item_num++;

  $menu->add('command', -label => 'Delete Account...',
             -underline => 0,
             -command => \&unimplemented_window);  
  $item_num++;

  $menu->add('separator');
  $item_num++;

  $menu->add('command',
             -label => 'Open Ledger',
             -underline => 5,
             -state => 'disabled',
             -command => \&open_ledger_for_current_selection,
             );
  $db_win_menu_items{"Edit\tOpen Ledger"} = [$menu, $item_num];
  $item_num++;
  

  $menu->add('separator');
  $item_num++;

  $menu->add('command', -label => 'Categories...',
             -underline => 5,
             -command => \&unimplemented_window,
             );
  $item_num++;

  $menubutton->configure(-menu => $menu);
  return $menu;
}
  

sub setup_main_file_menu {
  my($menubutton, $menu) = @_;  

  if($menu && $menu->Exists()) { $menu->destroy() }
  $menu = $menubutton->Menu(-tearoff => 0);
  
  $menu->add('command', -label => 'New Database...',
             -underline => 0,
             -command => \&db_new);
  $menu->add('command', -label => 'Load Database...',
             -underline => 0,
             -command => \&query_load_database);  
  $menu->add('command', -label => 'Save Database',
             -underline => 0,
             -command => \&db_save);
  $menu->add('command', -label => 'Save Database As...',
             -underline => 5,
             -command => \&db_save_as);    

  $menu->add('separator');

  # I'm sure these aren't working with the new code.
  $menu->add('command', -label => 'Import QIF File ...',
             -underline => 0,
             -command => \&db_import_qif,
             -state => 'disabled');
  $menu->add('command', -label => 'Export QIF File ...',
             -underline => 0,
             -command => \&db_export_qif,
             -state => 'disabled');
  
  $menu->add('separator');

  $menu->add('command', -label => 'Quit',
             -command => \&application_quit);
  $menu->entryconfigure(10, -accelerator => '<Meta-Q>');
  $menubutton->bind('all', '<Meta-q>' => \&application_quit);
  $menubutton->bind('all', '<Alt-q>' => \&application_quit);

  $menubutton->configure(-menu => $menu);
  return $menu;
}

sub setup_main_menubar {
  my($menubar) = @_;
  
  my %mb; 
  $mb{'file'} = 
    $menubar->Menubutton(-text => 'File', -underline => 0, -takefocus => 0);
  $mb{'edit'} = 
    $menubar->Menubutton(-text => 'Edit', -underline => 1, -takefocus => 0);
  $mb{'prefs'} = 
    $menubar->Menubutton(-text => 'Preferences', -underline => 1,
                         -takefocus => 0);
  $mb{'extern'} = 
    $menubar->Menubutton(-text => 'External', -underline => 1,
                         -takefocus => 0);
  if($main::cbb{'devel'}) {
    $mb{'devel'} = 
      $menubar->Menubutton(-text => 'Devel', -takefocus => 0);
  }  
  $mb{'help'} =
    $menubar->Menubutton(-text => 'Help', -underline => 0, -takefocus => 0);
  
  my @mbs = ($mb{'file'}, $mb{'edit'}, $mb{'prefs'}, $mb{'extern'});
  if($mb{'devel'}) { push @mbs, $mb{'devel'}; }
  my $mb;
  foreach $mb (@mbs) { $mb->pack(-side => 'left', -ipadx => 2); }
  $mb{'help'}->pack(-side => 'right', -ipadx => 1);

  return %mb
}

1
__END__

#------------------------------------------------------------------------------
# Setup the entry area
#------------------------------------------------------------------------------

sub setup_entry_area {
  my($parent) = @_;

  my $check_value = 1101;
  my $nicedate_value = '1997/05/04';
  my $desc_value = 'A sample transaction';
  my $category_value;
  my $debit_value = 12.21;
  my $credit_value = 19.44;
  my $status_value = 'x';
  
  my $comment_value = 'huh?';
  
  $parent->optionAdd('*font' => $main::cbb{'fixed-font'});
  
  my $filename = $main::cbb{'image-dir'} . '/mini-exclam.gif';  
  my $done_img = $parent->Photo("done", -file => $filename); 
  
  $filename = $main::cbb{'image-dir'} . '/mini-cross.gif';  
  my $cancel_img = $parent->Photo("cancel", -file => $filename); 

  my $line1 = $parent->Frame();
  my $line2 = $parent->Frame();

  $line1->pack(-side => 'top', -fill => 'x', -expand => 1);
  $line2->pack(-side => 'top', -fill => 'x', -expand => 1);
  
  my $check = $line1->Entry(-relief => 'sunken', -width => 5,
                             -textvariable => \$check_value);
  my $date = $line1->Entry(-relief => 'sunken', -width => 8,
                             -textvariable => \$nicedate_value);
  my $desc = $line1->Entry(-relief => 'sunken', -width => 15,
                             -textvariable => \$desc_value);
  my $debit = $line1->Entry(-relief => 'sunken', -width => 9,
                             -textvariable => \$debit_value);
  my $credit = $line1->Entry(-relief => 'sunken', -width => 9,
                             -textvariable => \$credit_value);
  my $status = $line1->Entry(-relief => 'sunken', -width => 1,
                             -textvariable => \$status_value);

  my $done = $line1->Button(-image => $done_img, -command => \&done_entering,
                            -takefocus => 0);
  cbb_set_balloon($done, 'Done entering <Enter>');
  
  $check->pack(-side => 'left');
  $date->pack(-side => 'left', -padx => 7);
  $desc->pack(-side => 'left');
  $debit->pack(-side => 'left', -padx => 7);
  $credit->pack(-side => 'left');
  $status->pack(-side => 'left', -padx => 6);
  $done->pack(-side => 'right');
  
  my $space = $line2->Label(-width => 15);
  my $comment =
    $line2->Entry(-width => 15, -relief => 'sunken',
                  -textvariable => \$comment_value);
  my $category =
    $line2->Entry(-width => 9, -relief => 'sunken',
                  -textvariable => \$category_value);

  my $cancel = $line2->Button(-image => $cancel_img,
                              -command => \&clear_entry_area,
                              -takefocus => 0);
  cbb_set_balloon($cancel, 'Cancel changes and start over <Meta-N>');
  
  $space->pack(-padx => 5.7, -side => 'left', );
  $comment->pack(-side => 'left');
  $category->pack(-padx => 7, -side => 'left');
  $cancel->pack(-side => 'right');
  
  # setup some bindings
  
  # Change the bindtags so the following bindings execute first
  # the break makes sure any other default bindings are skipped.
  $check->bindtags([$check, 'Entry', $main::mw, 'all']);
  $date->bindtags([$date, 'Entry', $main::mw, 'all']);
  
  # Don't use "all" bindings on this field so we can control what happens
  # with a <Tab> or <Shift-Tab>
  $desc->bindtags([$desc, 'Entry', $main::mw]);
  $category->bindtags([$category, 'Entry', $main::mw]);
  
  $check->bind('<KeyRelease-plus>', sub {
    print "FOO\n";
    $check_value = inc_check($check_value);
    $check->icursor('end');
    return;
  }); 
  
  $check->bind('<KeyRelease-equal>', sub {
    $check_value = inc_check($check_value);
    $check->icursor('end');
    return;
  });

  $check->bind('<KeyRelease-minus>', sub {
    $check_value = dec_check($check_value);
    $check->icursor('end');
    return;
  });
  
  $check->bind('<KeyRelease-underscore>', sub {
    $check_value = dec_check($check_value);
    $check->icursor('end');
    return;
  });

  $date->bind('<KeyRelease-plus>', sub {
    $nicedate_value = inc_date($nicedate_value);
    $date->icursor('end');
    return;
  });

  $date->bind('<KeyRelease-equal>', sub {
    $nicedate_value = inc_date($nicedate_value);
    $date->icursor('end');
    return;
  });

  $date->bind('<KeyRelease-minus>', sub {
    $nicedate_value = dec_date($nicedate_value);
    $date->icursor('end');
    return;
  });

  $date->bind('<KeyRelease-underscore>', sub {
    $nicedate_value = dec_date($nicedate_value);
    $date->icursor('end');
    return;
  });
}

  
  my($menubar, $headers, $transactions, $entry, $bar, $accts, $status) = 
    setup_containers($mw);

  # Setup menu bar
  $mw->optionAdd('*font' => $main::cbb{'menu-font'});

  my %menu_buttons = setup_menubar($menubar);
  
  my $file_menu = setup_file_menu($menu_buttons{'file'}, '');

  #setup_edit_menu .menubar
  #setup_functions_menu .menubar
  #setup_extern_menu .menubar
  
  my $devel_menu = '';
  if($menu_buttons{'devel'}) {
    $devel_menu = setup_devel_menu($menu_buttons{'devel'}, $devel_menu);
  }

  my $help_menu = setup_help_menu($menu_buttons{'help'});

  setup_headers($headers);
  setup_transaction_list($transactions);
  setup_entry_area($entry);

  if($main::cbb{'auto-hilite'}) {
    setup_auto_hilite();
  }

  setup_command_bar($bar);
  #setup_acct_listbox
  setup_status_line($status);
  
  $mw->update();
  $mw->deiconify();

sub load_acct_listbox {

    set a "no files found"
    set dname [file dirname $cbb(cur_file)]
    catch {set a [split [exec sh -c "cd $dname; echo *.cbb"] \ ]}
    .acct.list delete 0 [.acct.list size]
    foreach i $a {
	set b "no files found"
	if { $cbb(use_crypt) } {
	    catch { set b [exec sh -c "$cbb(decrypt) $cbb(crypt_code) < \
		    $dname/$i 2>/dev/null | grep '# Current Balance = '"]}
	} else {
	    catch {set b [exec grep "# Current Balance = " $dname/$i]}
	}
	regsub ".* " $b "" b
	regsub "\.cbb" $i "" i

	# get the account description
	puts $eng "get_cat_info \[$i\]"; flush $eng
	gets $eng desc
	
	if { "$desc" != "none" } {
	    set pieces [split $desc "\t"]
	    set desc [lindex $pieces 0]
	}

	.acct.list insert end [format "%-16s  %-37s  %11s" $i $desc $b]
    }

    update
}








#------------------------------------------------------------------------------
# Functions for the text box
#------------------------------------------------------------------------------

proc listAddTrans {line1 line2} {
    global cbb date start_pos

    set total [string range $line1 58 67]
    if { $cbb(debug) } { puts " --> $total" }
    set total [expr $total + 0]
    if { $cbb(debug) } { puts " --> $total" }
    if { $total < 0 } {
	set negative 1
    } else {
	set negative 0
    }

    set tmp_date [string range $line1 72 79]
    if { $cbb(debug) } { puts "$date - $tmp_date" }

    if { $negative } {
	.trans.list insert end "$line1\n" negcolor1
	.trans.list insert end "$line2\n" negcolor2
    } else {
	.trans.list insert end "$line1\n" color1
	.trans.list insert end "$line2\n" color2
    }

    if { $tmp_date <= $date } {
	set start_pos [listGetSize]
    }
}


proc listGetSize {} {
    scan [.trans.list index end] "%d.%d" line col
    return [expr $line - 2]
}


proc listGetCurTrans {} {
    global cbb

    if { "[.trans.list tag nextrange cbbSel 1.0 end ]" != "" } {
	scan [.trans.list tag nextrange cbbSel 1.0 end ] "%d.%d" line col
	# set line [expr $line + 1]
    } else {
	set line -1
    }

    if { $cbb(debug) } { puts $line }

    return $line
}


proc listHiliteTrans index {
    global cbb

    scan $index "%d.%d" line col


    .trans.list tag remove sel 1.0 end
    .trans.list tag remove cbbSel 1.0 end

    if { $line < 1 } {
	set line 1
    }

    if { [ expr $line / 2.0 ] != [ expr $line / 2] } {
	set index1 $line
	set index2 [ expr $line + 1 ]
    } else {
	set index1 [ expr $line - 1 ]
	set index2 $line 
    }
	
    .trans.list tag add cbbSel ${index1}.0 ${index2}.0lineend
    .trans.list see ${index1}.0
    .trans.list see ${index2}.0
}	

#------------------------------------------------------------------------------
# Setup the acct listbox and scrollbar
#------------------------------------------------------------------------------

proc setup_acct_listbox {} {
    global cbb yesno

    listbox .acct.list -width $cbb(list_width) -height $cbb(acctlist_height) \
	    -takefocus 0 -exportselection false \
	    -yscrollcommand ".acct.scroll set" -font $cbb(fixed_font)

    bind .acct.list <Double-Button> {
	if { "[.acct.list curselection]" != "" } {
	    if { [acctIsDirty] } {
		if { $cbb(auto_save) } {
		    acctSave
		} else {
		    cbbWindow.yesno "You have not saved your current changes.  \
			    Would you like to save before loading a new \
			    account?"
		    tkwait window .yesno
		    
		    if { "$yesno(result)" == "yes" } {
			acctSave
		    } elseif { "$yesno(result)" == "no" } {
		    } elseif { "$yesno(result)" == "cancel" } {
			return
		    }
		}
	    }
	    set file [.acct.list get [.acct.list curselection]].cbb
	    set dname  [file dirname $cbb(cur_file)]
	    regsub " .*$" $file "" file
	    acctLoadFile $dname/$file.cbb
	}
    }

    pack .acct.list -side left -fill both -expand 1

    scrollbar .acct.scroll -takefocus 0 -command ".acct.list yview" \
	    -relief flat
    pack .acct.scroll -side right -fill y -expand 1
}

proc load_acct_listbox {} {
    global cbb eng

    set a "no files found"
    set dname [file dirname $cbb(cur_file)]
    catch {set a [split [exec sh -c "cd $dname; echo *.cbb"] \ ]}
    .acct.list delete 0 [.acct.list size]
    foreach i $a {
	set b "no files found"
	if { $cbb(use_crypt) } {
	    catch { set b [exec sh -c "$cbb(decrypt) $cbb(crypt_code) < \
		    $dname/$i 2>/dev/null | grep '# Current Balance = '"]}
	} else {
	    catch {set b [exec grep "# Current Balance = " $dname/$i]}
	}
	regsub ".* " $b "" b
	regsub "\.cbb" $i "" i

	# get the account description
	puts $eng "get_cat_info \[$i\]"; flush $eng
	gets $eng desc
	
	if { "$desc" != "none" } {
	    set pieces [split $desc "\t"]
	    set desc [lindex $pieces 0]
	}

	.acct.list insert end [format "%-16s  %-37s  %11s" $i $desc $b]
    }

    update
}

#------------------------------------------------------------------------------
# Functions for entry area
#------------------------------------------------------------------------------

proc update_globals result {
    global cbb eng key date nicedate year month day check desc debit credit cat
    global nicecat com cleared total 

    set date ""; set year ""; set month ""; set day ""; set check ""
    set desc ""; set debit 0.00; set credit 0.00; set cat ""; set nicecat ""
    set com ""; set cleared ""; set total 0.00

    set pieces [split $result "\t"]
    set key [lindex $pieces 0]
    set date [lindex $pieces 1]
    if { [string length $date] == 6 } {
	set year "$cbb(century)[string range $date 0 1]"
	set month [string range $date 2 3]
	set day [string range $date 4 5]
    } else {
	set year [string range $date 0 3]
	set month [string range $date 4 5]
	set day [string range $date 6 7]
    }
    if { $cbb(date_fmt) == 1 } {
        set nicedate "$month/$day/[string range $year 2 3]"
    } else {
        set nicedate "$day.$month.[string range $year 2 3]"
    }
    set check [lindex $pieces 2]
    set desc [lindex $pieces 3]
    scan [lindex $pieces 4] "%f" debit
    scan [lindex $pieces 5] "%f" credit
    set debit [format "%.2f" $debit]; 
    set credit [format "%.2f" $credit]; 
    set cat [lindex $pieces 6]
    if { [string range $cat 0 0] == "|" } {
        set nicecat "-Splits-"
    } else {
    	set nicecat $cat
    }
    set nicecat [string range $nicecat 0 8]
    set com [lindex $pieces 7]
    set cleared [lindex $pieces 8]
    scan [lindex $pieces 9] "%f" total
}


# given a memorized transaction, update the relevant fields
proc update_from_mem result {
    global eng desc debit credit cat nicecat com 

    set desc ""; set debit 0.00; set credit 0.00; set cat ""; set nicecat ""
    set com ""; 

    set pieces [split $result "\t"]
    set desc [lindex $pieces 3]
    scan [lindex $pieces 4] "%f" debit
    scan [lindex $pieces 5] "%f" credit
    set cat [lindex $pieces 6]
    if { [string range $cat 0 0] == "|" } {
        set nicecat "-Splits-"
    } else {
    	set nicecat $cat
    }
    set nicecat [string range $nicecat 0 8]
    set com [lindex $pieces 7]

    set debit [format "%.2f" $debit]; 
    set credit [format "%.2f" $credit]; 
}

proc find_index_from_key args {
    global cbb
    # given a newkey, return the index of the first affected transaction

    set arglist [split $args]
    set cbb(index1) [lindex $arglist 0]
    set newkey [lindex $arglist 1]

    if { $cbb(debug) } { puts "find: cbb(index1) = $cbb(index1)  newkey = $newkey" }

    if { [expr $cbb(index1) / 2.0] == [expr $cbb(index1) / 2] } {
	set cbb(index1) [expr $cbb(index1) - 1]
    }

    if { $cbb(index1) < 1 } {
	set cbb(index1) 1
    }

    set line [.trans.list get $cbb(index1).0 $cbb(index1).0lineend ]
    set key [string range $line 72 end]

    if { $cbb(debug) } { puts "target = $newkey  current = $key" }

    if { [string compare "$newkey" "$key"] == -1 } {
	# we changed the date to something previous
	while { [expr [string compare "$newkey" "$key"] == -1 && $cbb(index1) > 0]} {
	    set cbb(index1) [expr $cbb(index1) - 2]
	    set line [.trans.list get $cbb(index1).0 $cbb(index1).0lineend ]
            set key [string range $line 72 end]
	    if { $cbb(debug) } { puts "target = $newkey  current = $key" }
	}
	return [expr $cbb(index1)]
    } else {
	# we changed the date to something forward or this is the trivial case
	return [expr $cbb(index1) - 2]
    }
}


proc find_index_from_date date {
    global cbb
    # given a date in the form yyyymmdd, return the index of the transaction
    # which is previous to the next higher date

    if { $cbb(debug) } { puts "find: date = $date" }

    set index [listGetSize]

    set line [.trans.list get ${index}.0 ${index}.0lineend ]
    set linedate [string range $line 72 79]

    if { $cbb(debug) } { puts "target = $date  current = $linedate" }

    while { [expr [string compare "$date" "$linedate"] == -1 && $index > 0]} {
	set index [expr $index - 2]
	set line [.trans.list get ${index}.0 ${index}.0lineend ]
	set linedate [string range $line 72 79]
	if { $cbb(debug) } { puts "target = $date  current = $linedate" }
    }

    return [expr $index]
}


proc update_rest args {
    global cbb key eng date nicedate year month day check desc debit credit cat
    global nicecat com cleared total

    set arglist [split $args]
    set cbb(index1) [lindex $arglist 0]
    set newkey [lindex $arglist 1]

    if { $cbb(debug) } { puts "update_rest: $cbb(index1) $newkey" }

    # save the current listbox view ...

    set yview_list [.trans.list yview]
    if { $cbb(debug) } { puts "Saving current view: $yview_list" }
    set yview_saved [lindex $yview_list 0]
    if { $cbb(debug) } { puts $yview_saved }

    # delete everything from the change forward, then rebuild our list from 
    # there

    set cbb(index1) [find_index_from_key $cbb(index1) $newkey]
    if { [expr $cbb(index1) < 0] } {
        set cbb(index1) 1
    }
    set cbb(index2) [expr $cbb(index1) + 1]

    if { $cbb(debug) } { puts "deleting from:  $cbb(index1).0 to end" }

    set line [.trans.list get $cbb(index1).0 $cbb(index1).0lineend ]
    set key [string range $line 72 end]
    .trans.list configure -state normal
    .trans.list delete $cbb(index1).0 end
    .trans.list configure -state disabled
 
    if { $cbb(debug) } { puts [string range $line 70 end] }
    if { $cbb(debug) } { puts "adding entries from $key to end" }

    if { $cbb(index1) == 1 } {
    	puts $eng "first_trans"; flush $eng
    } else {
        puts $eng "find_trans $key"; flush $eng
	.trans.list configure -state normal
	.trans.list insert end "\n"
	.trans.list configure -state disabled
    }
    .trans.list configure -state normal
    gets $eng result
    while { $result != "none" } {
        update_globals $result

        set checklen [string length $check]
        if { $checklen > 5 } {
	    set cutcheck [string range $check [expr $checklen - 5] end]
        } else {
	    set cutcheck $check
        }
        set cutdesc [string range $desc 0 14]
        set cutcom [string range $com 0 14]

	listAddTrans \
		[format "%5s  %-8s  %-15s  %9.2f  %9.2f  %-1s  %9.2f %14s" \
		$cutcheck $nicedate $cutdesc $debit $credit $cleared $total \
		$key] \
		[format "%5s  %-8s  %-15s  %-9s %39s" "" "" $cutcom $nicecat \
		$key] 

        # try keep the selection with the original transaction
        if { $key == $newkey } {
            set cbb(selected) [expr [listGetSize] - 1]
	    listHiliteTrans $cbb(selected).0
    	    set cbb(cur_date) $nicedate
 
	    if { "$check" != "" } {                                        
                set cbb(next_chk) $check                                  
            }
         }

        puts $eng "next_trans"; flush $eng
        gets $eng result
    }
    .trans.list configure -state disabled

    # now try to restore the current view ...

    .trans.list yview moveto $yview_saved
    set temp [listGetCurTrans]
    if { $temp >= 1 } {
	.trans.list see ${temp}.0
    } else {
	.trans.list see 1.0
    }
}


proc update_line args {
    global cbb key eng date nicedate year month day check desc debit credit cat
    global nicecat com cleared total

    set arglist [split $args]
    set cbb(index1) [lindex $arglist 0]
    set key [lindex $arglist 1]
    set temp [listGetCurTrans]
    if { $temp >= 1 } {
	set cbb(selected) $temp
    } else {
	set cbb(selected) 1
    }

    if { $cbb(debug) } { puts "update_line: $cbb(index1) $key" }

    # delete trans and re-insert

    set cbb(index2) [expr $cbb(index1) + 1]

    if { $cbb(debug) } { puts "deleting from:  $cbb(index1) to $cbb(index2)" }

    set line [.trans.list get $cbb(index1).0 $cbb(index1).0lineend ]
    set key [string range $line 72 end]
    .trans.list configure -state normal
    .trans.list delete $cbb(index1).0 [expr $cbb(index2) + 1].0
    .trans.list configure -state disabled
 
    if { $cbb(debug) } { puts "re-inserting entry" }

    puts $eng "find_trans $key"; flush $eng
    gets $eng result

    update_globals $result

    set checklen [string length $check]
    if { $checklen > 5 } {
	set cutcheck [string range $check [expr $checklen - 5] end]
    } else {
	set cutcheck $check
    }
    set cutdesc [string range $desc 0 14]
    set cutcom [string range $com 0 14]

    .trans.list configure -state normal
    .trans.list insert $cbb(index1).0 \
           [format "%5s  %-8s  %-15s  %9.2f  %9.2f  %-1s  %9.2f %14s\n" \
           $cutcheck $nicedate $cutdesc $debit $credit $cleared $total \
	   $key] color1
    .trans.list insert $cbb(index2).0 \
           [format "%5s  %-8s  %-15s  %-9s %39s\n" "" "" $cutcom $nicecat \
	   $key] color2
    .trans.list configure -state disabled

    listHiliteTrans $cbb(selected)
}


proc clear_entry_area {} {
    global cbb key eng date nicedate year month day check desc debit credit cat
    global nicecat com cleared total

    set key ""; set date ""; set year ""; set month ""; set day ""
    set check ""; set desc ""; set debit 0.00; set credit 0.00; set cat ""
    set nicecat ""; set com ""; set cleared ""; set total 0.00

    if { "$cbb(cur_date)" != "" } {
    	set nicedate $cbb(cur_date)
    } else {
        # set nicedate [fmtclock [getclock] "%m/%d/%y"]
    	puts $eng "nice_date $cbb(date_fmt)"; flush $eng
    	gets $eng nicedate
	set cbb(cur_date) $nicedate
    }
    # set date [fmtclock [getclock] "%Y%m%d"]
    puts $eng "raw_date"; flush $eng
    gets $eng date

    # get internal sdate, if sdate is defined
    if { $cbb(sdate) != "" } {
	puts $eng "start_date $cbb(sdate)"; flush $eng
	gets $eng cbb(int_sdate)
    }

    if { $cbb(debug) } { puts $nicedate; puts $date }

    set cbb(no_more_mem) 0

    focus .entry.line1.check
}


proc update_entry_area lineindex {
    global cbb key eng date nicedate year month day check desc debit credit cat
    global nicecat com cleared total

    scan $lineindex "%d.%d" item col
    set item [expr $item + 1]

    set cbb(no_more_mem) 1

    set cbb(selected) $item

    if { [expr $item / 2.0] == [expr $item / 2] } {
	set cbb(index1) $item
	set cbb(index2) [expr $item + 1]
    } else {
	set cbb(index1) [expr $item - 1]
	set cbb(index2) $item
    }

    set line [.trans.list get $cbb(index1).0 $cbb(index1).0lineend ] 
    set key [string range $line 72 end]

    puts $eng "find_trans $key"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts $result }

    if { $result != "none" } {
    	update_globals $result
    }

    # warn if about to edit a closed transaction
    if { "$cleared" == "x" } {
	cbbWindow.ok "You are about to edit a ``Closed'' transaction.  \
		Hopefully you know what you are doing."
	tkwait window .ok
    }

    # warn if about to edit a transfer transaction
    if { "[string range $cat 0 0]" == "\[" } {
	cbbWindow.ok "Notice:  You are about to edit a ``Transfer'' transaction.  \
		The corresponding transaction in the file ``$cat''  \
		will also be updated."
	tkwait window .ok
    }


    focus .entry.line1.check
}


proc done_entering {} {
    global cbb yesno key eng date nicedate year month day check desc debit 
    global credit cat nicecat com cleared total addcat

    if { $cbb(debug) } { puts "Done entering ..." }

    # check for a valid file
    if { "$cbb(cur_file)" == "noname.cbb" } {
	cbbWindow.ok "You must Make or Load an Account First."
	tkwait window .ok
	return
    } elseif { "$cbb(cur_file)" == ""} {
	cbbWindow.ok "You must Make or Load an Account First."
	tkwait window .ok
	return
    }

    # we now have something to save
    acctSetDirty

    # do some consistency checking here
    if { "$desc" == "" } {
	cbbWindow.yesno "You have not entered anything in the description \
		field.  Would you like to continue?"
	tkwait window .yesno
	if { "$yesno(result)" != "yes" } {
	    return
	}
    }

    # pad date if needed
    if { $cbb(date_fmt) == 1 } {
	set pieces [split $nicedate /]
        set month [lindex $pieces 0]
        set day [lindex $pieces 1]
    } else {
 	set pieces [split $nicedate .]
	set day [lindex $pieces 0]
        set month [lindex $pieces 1]
    }

    if { [lindex $pieces 2] != "" } {
	set year [lindex $pieces 2]
    } else {
	# get last entered year
	set pieces [split $cbb(cur_date) /]
	set year [lindex $pieces 2]
    }
    set month [pad $month]
    set day [pad $day]
    set year [pad $year]
    if { [string length $year] == 2 } {
	set year $cbb(century)$year
    }
    if { $cbb(date_fmt) == 1 } {
        if { $cbb(debug) } { puts "$month/$day/$year" }
        set nicedate "$month/$day/$year"
    } else {
        if { $cbb(debug) } { puts "$day.$month.$year" }
        set nicedate "$day.$month.$year"
    }

    if { "[string range $cat 0 0]" != "|" } {
        # if not a split, try to match category
        puts $eng "find_cat $cat"; flush $eng
        gets $eng result
        if { "$result" != "none" } {
            set cat $result
        } elseif { "$cat" == "" } {
	    cbbWindow.yesno "You have not entered anything in the category \
		    field.  Would you like to continue?"
	    tkwait window .yesno
	    if { "$yesno(result)" != "yes" } {
	        return
	    }
	} else {
	    set addcat(cat) $cat
	    set addcat(mode) "missing"
            cbbWindow.newcat
            tkwait window .newcat
	    if { $cbb(debug) } { puts $addcat(result) }
	    if { "$addcat(result)" != "yes" } {
		return
	    }
        }
    }

    # verify cleared field
    set cleared [string range $cleared 0 0]
    if { "$cleared" == "x" } {
        # ok
    } elseif { "$cleared" == "*" } {
        # ok
    } elseif { "$cleared" == "?" } {
        # ok 
    } elseif { "$cleared" == "" } {
        # ok 
    } else {
        set cleared ""
    }

    if { "$key" == "" } {
    	# new entry ... insert
	if { "[string range $cat 0 0]" == "\[" } {
	    # transfer transaction
	    puts $eng "create_xfer $year$month$day\t$check\t$desc\t$debit\t$credit\t$cat\t$com\t$cleared\t0.00"
	} else {
	    # normal transaction
	    puts $eng "create_trans $year$month$day\t$check\t$desc\t$debit\t$credit\t$cat\t$com\t$cleared\t0.00"
	}
        flush $eng
        gets $eng result
        if { $cbb(debug) } { puts "result:  create_trans $result" }

        if { "$result" == "error" } {
	    cbbWindow.ok "Your transaction entry returned an error:  \
		    '$result'.  If it was a transfer transaction, it probably \
		    couldn't find the ``to'' account.  Things could \
		    potentially be in an unsettled state.  You should \
		    probably save everything and manually make sure things \
		    are ok." 
        } else {
            undoRegister "insert $result"
        }
	update_rest [listGetSize] [string range $result 0 10]
    } else {
	if { [string length "$key"] != 11 } {
	    # try to make sure we have a valid key
	    cbbWindow.ok "Bad key value '$key'.  This transaction is aborted."
	    tkwait window .ok
	    set key ""
	    return
	}
	if { [string first "-" "$key"] != 8 } {
	    # try to make sure we have a valid key
	    cbbWindow.ok "Bad key value '$key'.  This transaction is aborted."
	    tkwait window .ok
	    set key ""
	    return
	}

	# if { "[string range $cat 0 0]" == "\[" } {
	#     cbbWindow.ok "You have edited a ``Transfer'' transaction.  The \
	#	    corresponding transaction in the file ``$cat'' cannot \
	#	    currently be changed.  You must do this manually."
	#     tkwait window .ok
	# }

        # first record the official version of this transaction so we can be
        # able to undelete it later
        puts $eng "find_trans $key"; flush $eng
        gets $eng origresult

        # updating an existing entry
	if { "[string range $cat 0 0]" == "\[" } {
	    # transfer transaction
	    puts $eng "update_xfer $key\t$year$month$day\t$check\t$desc\t$debit\t$credit\t$cat\t$com\t$cleared\t0.00"
	} else {
	    puts $eng "update_trans $key\t$year$month$day\t$check\t$desc\t$debit\t$credit\t$cat\t$com\t$cleared\t0.00"
	}
        flush $eng
        gets $eng result

	if { "$cbb(index1)" == "" } {
	    set cbb(index1) [listGetSize]
	}
	update_rest $cbb(index1) [string range $result 0 10]
        undoRegister "edit [string range $result 0 10]\t$origresult"
    }

    # try keep the entry area in sync with the selection
    goto $cbb(selected)

    clear_entry_area
}


proc delete_trans lineindex {
    global cbb yesno eng cat cleared

    scan $lineindex "%d.%d" item col
    # set item [expr $item - 1]

    acctSetDirty

    if { [expr $item / 2.0] != [expr $item / 2] } {
	set cbb(index1) $item
	set cbb(index2) [expr $item + 1]
    } else {
	set cbb(index1) [expr $item - 1]
	set cbb(index2) $item
    }

    set line [.trans.list get $cbb(index1).0 $cbb(index1).0lineend ]
    set key [string range $line 72 end]

    # first record the official version of this transaction so we can be
    # able to undelete it later
    puts $eng "find_trans $key"; flush $eng
    gets $eng result
    update_globals $result
    undoRegister "delete $result"

    if { "[string range $cat 0 0]" == "\[" } {
	cbbWindow.ok "Notice:  You are deleting a ``Transfer'' transaction.  \
		The corresponding transaction in the file ``$cat'' will \
		also be deleted."
	tkwait window .ok
    }

    if { "$cleared" == "x" } {
	cbbWindow.yesno "You are deleting a ``Closed'' transaction.  Continue \
		with delete?"
	tkwait window .yesno
	
	if { "$yesno(result)" == "yes" } {
	} elseif { "$yesno(result)" == "no" } {
	    return
	} elseif { "$yesno(result)" == "cancel" } {
	    return
	}
    }

    if { "[string range $cat 0 0]" == "\[" } {
	puts $eng "delete_xfer $key"; flush $eng
    } else {
	puts $eng "delete_trans $key"; flush $eng
    }
    gets $eng result
    if { $cbb(debug) } { puts "deleting:  $result" }

    update_rest $cbb(index1) $key

    goto $cbb(index1)

    clear_entry_area
}


#------------------------------------------------------------------------------
# Miscellaneous functions
#------------------------------------------------------------------------------

proc goto line {
    global cbb

    if { $cbb(debug) } { puts "Size = [listGetSize]  Goto = $line" }

    set max [expr [listGetSize] - 1]
    if { $line > $max } {
	set line $max
    }

    if { [expr $line / 2.0] == [expr $line / 2] } {
        set line [expr $line - 1]
    }

### .trans.list see $line
### .trans.list see [expr $line + 1]
### .trans.list selection clear 0 end
### .trans.list selection set $line $line
    listHiliteTrans $line
}


proc acctSetClean { } {
    global cbb

    set cbb(clean) 1
    if { [winfo exists .bar.save] } {
	.bar.save configure -text "Save"
	cbb_set_balloon .bar.save "Save the current file (file is clean)"
    }

    # update account listbox
    if { [info exists cbb(cur_file)] } {
        load_acct_listbox
    }
}


proc acctSetDirty { } {
    global cbb
    
    set cbb(clean) 0
    if { [winfo exists .bar.save] } {
	.bar.save configure -text "Save!"
	cbb_set_balloon .bar.save \
		"Save the current file (file has been changed)"
    }
}


proc acctIsDirty { } {
    global cbb

    return [expr 1 - $cbb(clean)]
}




  



proc setup_edit_menu m {


  #$menu->add('cascade', -label => 'Preferences',
  #           -underline => 0,
  #           -menu => setup_file_prefs_menu($menu));

  $menu->add('separator');



    set me $m.edit.menu

    menu $me -tearoff 0

    $me add command -label 'Undo' -underline 0 -command { undoAction }
    $me entryconfig 0 -accelerator <Meta-U>
    bind all <Meta-u> { undoAction }
    bind all <Alt-u> { undoAction }

    $me add separator

    $me add command -label 'New Transaction' \
	    -underline 0 -command { clear_entry_area }
    $me entryconfig 2 -accelerator <Meta-N>
    bind all <Meta-n> { clear_entry_area }
    bind all <Alt-n> { clear_entry_area }

    $me add command -label 'Edit Transaction' -underline 0 \
	    -command { 
	if { [listGetCurTrans] >= 1 } {
	    update_entry_area [listGetCurTrans].0
	}
    }
    $me entryconfig 3 -accelerator <Meta-E>
    bind all <Meta-e> { 
	if { [listGetCurTrans] >= 1 } {
	    update_entry_area [listGetCurTrans].0
	}
    }
    bind all <Alt-e> { 
	if { [listGetCurTrans] >= 1 } {
	    update_entry_area [listGetCurTrans].0
	}
    }

    $me add command -label 'Delete Transaction' -underline 0 \
	    -command { 
	if { "[.trans.list tag nextrange sel 1.0 end ]" != "" } {
	    delete_trans [.trans.list tag nextrange sel 1.0 end ]
	}
    }

    $me add separator

    $me add command -label 'Open Category Splits ... ' -underline 0 \
	    -command { 
	cbbWindow.splits
	tkwait window .splits
    }
    $me entryconfig 6 -accelerator <Meta-S>
    bind all <Meta-s> { 
	cbbWindow.splits
	tkwait window .splits
    }
    bind all <Alt-s> { 
	cbbWindow.splits
	tkwait window .splits
    }
}


proc setup_functions_menu m {
    global cbb

    set mf $m.functions.menu

    menu $mf -tearoff 0

    $mf add cascade -label 'Goto' -underline 0 -menu $mf.goto

    $mf add separator

    $mf add command -label 'Category List ...' \
	    -underline 0 -command { cbbWindow.catwin }
    $mf entryconfig 0 -accelerator <Meta-C>
    bind all <Meta-c> { cbbWindow.catwin }
    bind all <Alt-c> { cbbWindow.catwin }

    $mf add command -label 'Add Default Categories' \
	    -underline 0 -command { catImportDefaults }

    $mf add separator

    $mf add command -label 'Balance ...' -underline 0 -command { balance }

    $mf add separator

    $mf add command -label 'Reports ...' -underline 0 -command { 
	set report(type) 'report'
	set report(cur_file) $cbb(cur_file)
	set report(tmp_file) '/tmp/cbb-tmp-[pid].cbb'

	if { $cbb(debug) } { 
	    puts "Temp file is $report(tmp_file) -- saving ..."
	}
	# tell engine to save transactions
	puts $eng "auto_save_trans $report(tmp_file)"; flush $eng
	if { $cbb(debug) } { puts 'No result should be returned.' }

	cbbWindow.rep
    }

    $mf add command -label 'Graphs ...' -underline 0 -command { 
	set report(type) 'graph'
	set report(cur_file) $cbb(cur_file)
	set report(tmp_file) "/tmp/cbb-tmp-[pid].cbb"

	if { $cbb(debug) } { 
	    puts "Temp file is $report(tmp_file) -- saving ..."
	}
	# tell engine to save transactions
	puts $eng "auto_save_trans $report(tmp_file)"; flush $eng
	if { $cbb(debug) } { puts 'No result should be returned.' }

	cbbWindow.rep
    }
}


proc setup_extern_menu m {
    global lib_path

    set me $m.extern.menu

    menu $me -tearoff 0

    set filehandle [open "$lib_path/extern.conf" r]
    while { [gets $filehandle line] >= 0 } {
	if { "[string range $line 0 0]" == "-" } {
            $me add separator
	} elseif { "[string range $line 0 0]" == "#" } {
	    # ignore this line
	} elseif { [string length $line] < 3  } {
	    # ignore this line
        } else {
	    set pieces [split $line "\t"]
	    set name [lindex $pieces 0]
	    set prog [lindex $pieces 1]
            $me add command -label $name -command "launch_extern \"$prog\""
	}
    }
    close $filehandle
}


proc launch_extern { prog } {
    global report cbb eng

    if { $cbb(safe_mode) == 1 } {
        cbbWindow.ok "Cannot launch external scripts when running in 'safe' \
		mode." 
	tkwait window .ok
        return
    }

    set report(tmp_file) "/tmp/cbb-tmp-[pid].cbb"
    if { $cbb(debug) } { puts "Temp file is $report(tmp_file) -- saving ..." }
    # tell engine to save transactions
    puts $eng "auto_save_trans $report(tmp_file)"; flush $eng
    if { $cbb(debug) } { puts "No result should be returned." }

    regsub -all "%a" $prog $cbb(cur_file) prog
    regsub -all "%t" $prog $report(tmp_file) prog

    if { $cbb(debug) } { puts "exec $prog &" }
    eval exec $prog &
}





proc setup_functions_goto_menu m {
    set mfg $m.functions.menu.goto

    menu $mfg -tearoff 0

    $mfg add command -label "Home" -underline 0 -command { goto 0 }
    $mfg entryconfig 0 -accelerator <Meta-Home>
    $mfg add command -label "Today" -underline 0 -command { 
	puts $eng "raw_date"; flush $eng
	gets $eng tmpdate
	goto [find_index_from_date $tmpdate] 
    }
    $mfg add command -label "End" -underline 0 -command { 
	goto [expr [listGetSize] - 1]
    }
    $mfg entryconfig 2 -accelerator <Meta-End>

    $mfg add separator

    $mfg add command -label "Page Up" -underline 5 \
     	    -command {
	goto [expr [.trans.list curselection] - ($cbb(list_height) - 4)]
    }
    $mfg entryconfig 4 -accelerator <Prior>
    $mfg add command -label "Page Down" -underline 5 \
     	    -command { 
	goto [expr [.trans.list curselection] + ($cbb(list_height) - 4)]
    }
    $mfg entryconfig 5 -accelerator <Next>

    $mfg add separator

    $mfg add command -label "Up" -underline 1 \
     	    -command {
	goto [expr [.trans.list curselection] - 2]
    }
    $mfg entryconfig 7 -accelerator <Up>
    $mfg add command -label "Down" -underline 1 \
     	    -command { 
	goto [expr [.trans.list curselection] + 2]
    }
    $mfg entryconfig 8 -accelerator <Down>
}


# ----------------------------------------------------------------------------
# $Log: db-win.pl,v $
# Revision 1.2  1998/02/14 01:02:21  rlb
# comments mod
#







# Old Tcl log material:
#
# Revision 2.27  1997/06/12 21:53:14  curt
# Applied more patches from Martin Schenk <schenkm@ping.at>.  His changes
# greatly improved and developed the preferences menu so it is now actually
# usable to set preferences.
# Current data file is now saved in ~/.cbbcur.tcl
#
# Revision 2.26  1997/05/06 01:00:25  curt
# Added patches contributed by Martin Schenk <schenkm@ping.at>
# - Default to umask of 066 so .CBB files get created rw by owner only
# - Added support for pgp encrypting data files
# - Added support for displaying only recent parts of files (avoids
#   waiting to load in lots of old txns you don't currently need.)
# - Added a feature to "cache" whole accounts in the perl engine so
#   that switching between accounts can be faster.
# - The above options can be turned on/off via the preferrences menu.
#
# Revision 2.25  1997/04/12 01:15:26  curt
# Display current balance rather than ending balance in the account list
# box.  This makes a difference if you like to insert future transactions.
#
# Revision 2.24  1997/04/11 20:26:53  curt
# $index1, $index2 wrapped up so they are now $cbb(index1), and $cbb(index2)
#
# Revision 2.23  1997/04/04 22:50:20  curt
# Tweaked the previous fix.
#
# Revision 2.22  1997/04/04 18:44:48  curt
# Commented out some old menubar baggage.
# Fixed a harmless, but annoying bug that caused a pop up tk error message
# on occasion when manipulating the account list box.
#
# Revision 2.21  1997/04/03 03:54:49  curt
# Enable auto save without prompting when switching to a new account.
# Contributed by Jonathan I. Kamens <jik@kamens.brookline.ma.us>
#
# Revision 2.20  1997/04/03 03:42:20  curt
# Heading, line 2 color was hardcoded rather than honoring the header color
# variable.
#
# Revision 2.19  1997/03/04 03:22:39  curt
# Fixed bug which showed up when hitting '+' in a blank check # field after
# an account had been loaded, but before any transactions with a check number
# had been added.
#
# Revision 2.18  1997/02/19 18:08:06  curt
# Fixed a bug with long check numbers (> 5 char) in the balance window.
# Nailed a bug which caused cbb to forget the last check # entered when
# using +/- in the check number field.
#
# Revision 2.17  1997/01/28 03:25:24  curt
# Force strict scoping in all perl scripts.
#
# Revision 2.16  1997/01/16 19:15:42  curt
# Miscellaneous interface tweaks.
#
# Revision 2.15  1997/01/10 22:03:32  curt
# Transfer fixups and a few other misc. fixes contributed by
#   Lionel Mallet <Lionel.Mallet@sophia.inria.fr>
#
# Revision 2.14  1997/01/09 04:03:06  curt
# Allow user to specify colors of alternating transaction lines.
#
# Revision 2.13  1997/01/09 03:56:59  curt
# Added contrib script loan.pl.
# User sizable account list.
# Removed some old gnuplot baggage from install.pl.
#
# Revision 2.12  1997/01/02 04:38:36  curt
# Changes over the 1996 holidays:
#   - Converted listbox to text widget.  This allows us to do nice
#     things with alternating background colors, highliting, red
#     negative numbers, etc.
#   - Negative transactions are now drawn in red.
#   - Added a Goto <Today> option.
#   - <Home> & <End> were double bound.  Now, listbox can be traversed with
#     <Meta-Home> and <Meta-End>
#
# Revision 2.11  1996/12/17 20:15:44  curt
# Version incremented to 0.70.
# No longer save running total in .cbb files.
# Miscellaneous tweaks.
#
# Revision 2.10  1996/12/17 14:54:00  curt
# Updated copyright date.
#
# Revision 2.9  1996/12/16 04:18:21  curt
# Continuing the great overhaul of December 1996.
#
# Revision 2.8  1996/12/14 17:15:23  curt
# The great overhaul of December '96.
#
# Revision 2.7  1996/12/13 01:26:59  curt
# Worked on getting reports.tcl to work smoothly.
#
# Revision 2.6  1996/12/11 18:33:38  curt
# Ran a spell checker.
#
# Revision 2.5  1996/12/11 04:32:31  curt
# Several minor tweaks.
#
# Revision 2.4  1996/12/11 01:03:44  curt
# Added balloon help support.
#
# Revision 2.3  1996/12/09 14:38:23  curt
# Added "ok" and "cancel" buttons to entry area.  Tweaked check# and data +/-
# bindings.
#
# Revision 2.2  1996/12/08 07:40:01  curt
# Rearranged quite a bit of code.
# Put most global variables in cbb() structure.
#
# Revision 2.1  1996/12/07 20:38:16  curt
# Renamed *.tk -> *.tcl
#
# Revision 2.10  1996/10/03 04:49:00  curt
# Fixed an inconsistency in &raw_date() in common.pl (with how it was
# called.)
#
# Version now is 0.67-beta-x
#
# Revision 2.9  1996/10/03 03:57:37  curt
# Adjusted spacing of 2nd entry line so fields better align with the first.
#
# Revision 2.8  1996/10/02 19:37:20  curt
# Replaced instances of hardcoded century (19) with a variable.  We need to
# know the current century in cases where it is not provided and it is
# assumed to be the current century.  Someday I need to figure out how
# to determine the current century, but I have a couple of years to do it. :-)
#
# I still need to fix conf-reports and reports.pl
#
# Revision 2.7  1996/10/01 20:25:38  curt
# Added better handling of unknown category when trying to "commit" a
# transaction.
#
# Revision 2.6  1996/09/30 15:14:38  curt
# Updated CBB URL, and hardwired wish path.
#
# Revision 2.5  1996/09/25 17:45:42  curt
# Revamped tab completions in description and category fields.
# Fixed a problem with autohiliting.  When tabbing to a blank field, we used
# to leave the previous hilited field hilited.
#
# Revision 2.4  1996/08/29 14:22:33  curt
# <Meta-Tab> changed to <Control-Tab> in desc field.
#
# Revision 2.3  1996/07/13 02:57:49  curt
# Version 0.65
# Packing Changes
# Documenation changes
# Changes to handle a value in both debit and credit fields.
#
# Revision 2.2  1996/03/03  00:16:13  curt
# Modified Files:  cbb categories.pl wrapper.pl file.tk main.tk menu.tk
#   Added an account list at the bottom of the screen.  Thanks to:
#   Cengiz Alaettinoglu <cengiz@ISI.EDU> for this great addition.
#
# Revision 2.1  1996/02/27  05:35:48  curt
# Just stumbling around a bit with cvs ... :-(
#
# Revision 2.0  1996/02/27  04:43:01  curt
# Initial 2.0 revision.  (See "Log" files for old history.)

