#
#  'CBB' -- Check Book Balancer
#
#   help.pl -- help routines
#
#  Written by Curtis Olson.  Started December 7, 1996.
#
#  Copyright (C) 1994 - 1997  Curtis L. Olson  - curt@sledge.mn.org
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Id: help.pl,v 1.1 1998/01/24 02:26:30 rlb Exp $
# (Log is kept at end of this file)


#------------------------------------------------------------------------------
# Help menu procedures
#------------------------------------------------------------------------------

my $about_win;

sub open_or_raise_about_win {
  my($name) = @_;
  
  if($about_win && $about_win->Exists()) {
    $about_win->raise();
    return;
  }

  $about_win = $main::mw->Toplevel();
  $about_win->optionAdd('*font', $pref_msg_text_font);
  $about_win->title("About $name");
  $about_win->iconname("About $name");

  my $main_frame = 
    $about_win->Frame(-borderwidth =>  2, -relief =>  'raised');

  my $frame_a = $main_frame->Frame(-borderwidth => 2);
  my $frame_d = $main_frame->Frame(-borderwidth => 2);

  my $button_a =
    $frame_a->Button(-bitmap => '@' . $pref_image_dir . '/' .
                     'author.xbm',
                     -relief => 'ridge');
  
  my @strings = 
    ('The wise man saves for the future ...',
     '... the fool spends everything he gets.',
     'Proverbs 21:20',
     "``$name'' -- a Check Book Balancer for X, " . $main::version,
     'Copyright (C) 1994 - 1997  Curtis L. Olson',
     'curt@sledge.mn.org',
     'http://www.menet.umn.edu/~curt');
  
  my @labels = map {
    $frame_a->Label(-text => $_);
  } @strings;
 
  my $dismiss =
    $frame_d->Button(-text => 'Dismiss',
                     -command => sub { $about_win->destroy(); },
                     -font => $pref_button_font);
  
  set_balloon($dismiss, 'This is a test of the emergency broadcast system.');

  $main_frame->pack(-side => 'top',  -fill => 'both',  '-expand' => 1);
  $frame_d->pack(-side => 'bottom',  -fill => 'x');
  $frame_a->pack(-side => 'top', -fill => 'both', -expand => 1);

  $button_a->pack(-padx => 8, -pady => 8, -side => 'left',
                  -fill => 'both', -expand => 1);

  map {
    $_->pack(-ipadx => '4m', -side => 'top', -fill => 'both', -expand => 1);
  } @labels;

  $dismiss->pack(-side => 'bottom', -fill => 'x', -expand => 1,
                 -padx => 8, -pady => 4);
}


my $help_win;

sub open_or_raise_help_win {
  
  if($help_win && $help_win->Exists()) {
    $help_win->raise();
    return;
  }
  
  $help_win = $main::mw->Toplevel();
  $help_win->optionAdd('*font', $pref_msg_text_font);

  $help_win->title("Help (Text)");
  $help_win->iconname("Help");

  my $main_frame = 
    $help_win->Frame(-borderwidth =>  2, -relief =>  'raised');

  my $frame_l = $main_frame->Frame(-borderwidth => 2);
  my $frame_d = $main_frame->Frame(-borderwidth => 2);

  $main_frame->pack(-side => 'top', -fill => 'both', -expand => 1);
  $frame_d->pack(-side => 'bottom', -fill => 'x');
  $frame_l->pack(-side => 'top', -fill => 'both', -expand => 1);

  my $list = $frame_l->Text(-width => 80, -height => 40, -relief => 'sunken',
                            -font => $pref_fixed_font);	
  my $scroll = $frame_l->Scrollbar(-command => [$list,  'yview'],
                                   -relief => 'flat');
  
  $list->configure(-yscrollcommand => [ $scroll => 'set' ]);
  
  $scroll->pack(-side => 'right', -fill => 'y', -expand => 0);
  $list->pack(-side => 'left', -fill => 'both', -expand => 1);

  $help_win->bind('<KeyPress-Down>', sub {
    $list->yview('scroll' => 1, 'units'); });
  $help_win->bind('<Control-KeyPress-n>', sub {
    $list->yview('scroll' => 1, 'units'); });
  $help_win->bind('<KeyPress-n>', sub { 
    $list->yview('scroll' => 1, 'units'); });
  $help_win->bind('<KeyPress-j>', sub {
    $list->yview('scroll' => 1, 'units'); });
  
  $help_win->bind('<KeyPress-Up>' , sub {
    $list->yview('scroll' => -1, 'units'); });
  $help_win->bind('<Control-KeyPress-p>' , sub {
    $list->yview('scroll' => -1, 'units'); });
  $help_win->bind('<KeyPress-p>' , sub {
    $list->yview('scroll' => -1, 'units'); });
  $help_win->bind('<KeyPress-k>' , sub {
    $list->yview('scroll' => -1, 'units'); });

    $help_win->bind('<KeyPress-Next>' , sub {
      $list->yview('scroll' => 1, 'pages'); });
  $help_win->bind('<Control-KeyPress-v>' , sub {
    $list->yview('scroll' => 1, 'pages'); });
  
  $help_win->bind('<KeyPress-Prior>' , sub {
    $list->yview('scroll' => -1, 'pages'); });
  $help_win->bind('<Alt-KeyPress-v>' , sub {
    $list->yview('scroll' => -1, 'pages'); });
  
  my $dismiss =
    $frame_d->Button(-text => 'Dismiss',
                     -font => $pref_button_font,
                     -command => sub { $help_win->destroy(); });
  
  $dismiss->pack(-side => 'top', -expand => 1, -fill => 'x',
                 -padx => 8, -pady => 4);

  my $f = new IO::File;

  $f->open('<' . $pref_doc_dir . '/cbb-man.txt');
  my @lines = <$f>;
  $f->close();
  
  $list->insert('end', @lines);

  $list->configure(-state => 'disabled');
}


sub menubutton_add_help_menu {
  my($menubutton) = @_;
  
  my $menu = $menubutton->Menu(-tearoff => 0);
  
  $menu->add('command', -label => "About $0...",
             -underline => 0,
             -command => sub { open_or_raise_about_win($0); });

  $menu->add('separator');
      
  $menu->add('command', -label => 'Online HTML Manual...',
             -underline => 0,
             -command => sub {
               if(!$pref_safe) {
                 if($pref_web_browser eq 'netscape') {
                   # ... so they want to use nutscape ...
                   if(system('xwininfo -name "Netscape"')) {
                     # no existing netscape running
                     system($pref_web_browser . ' ' .
                            $pref_doc_dir . 
                            '/cbb-man/cbb-man.html &');
                   } else {
                     # A running copy of netscape exists
                     system($pref_web_browser .
                            ' -remote openURL(' . 
                            $pref_doc_dir . 
                            '/cbb-man/cbb-man.html) &');
                   }
                 } elsif($pref_web_browser eq 'lynx') {
                   system('xterm -sb -e ' . $pref_web_browser . ' ' .
                          $pref_doc_dir . 
                          '/cbb-man/cbb-man.html &');
                 } else {
                   system($pref_web_browser . ' ' .
                          $pref_doc_dir . 
                          '/cbb-man/cbb-man.html &');
                 }
               } else {
                 my $win = 
                   cbbWindow_ok('The online manual can be read from ' . 
                                'the CBB web page at: ' . 
                                'http://www.menet.umn.edu/~curt/cbb/');
                 $win->waitWindow();
               }
             });
  
  $menu->add('command', -label => 'Online Text Manual...',
             -command => \&open_or_raise_help_win);
  $menubutton->configure(-menu => $menu);
}




1;
__END__



# ----------------------------------------------------------------------------
# $Log: help.pl,v $
# Revision 1.1  1998/01/24 02:26:30  rlb
# Initial CVS submission.
#
# Revision 2.6  1997/04/23 18:10:02  curt
# Changed "clolson" -> "curt"
#
# Revision 2.5  1997/03/05 18:58:33  curt
# Added additional bindings to scrolling lists: categories list, balance
# window list, text help, and report list.
#
# Revision 2.4  1997/01/16 19:15:40  curt
# Miscellaneous interface tweaks.
#
# Revision 2.3  1996/12/17 14:53:57  curt
# Updated copyright date.
#
# Revision 2.2  1996/12/16 04:18:18  curt
# Continuing the great overhaul of December 1996.
#
# Revision 2.1  1996/12/08 07:37:49  curt
# Initial revision.
#
