#
#  'CBB' -- Check Book Balancer
#
#   init.pl -- Initialization routines.
#
#  Written by Curtis Olson.  Started December 7, 1996.
#
#  Copyright (C) 1994 - 1997  Curtis L. Olson  - curt@sledge.mn.org
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# $Id: init.pl,v 1.2 1998/01/24 02:21:25 rlb Exp $
# (Log is kept at end of this file)

use strict;
use Tk;

# We know where lib-dir is by now, so use it...
use vars qw(@source_files);
@source_files = grep { !/init\.pl/ } glob($pref_src_dir . '/*.pl');

# ----------------------------------------------------------------------------
# Draw splash screen on startup
# ----------------------------------------------------------------------------

sub cbb_splash {                                              
  my($mw, $filename)= @_;

  my $x = $mw->Toplevel(-bd => 3, -relief => 'raised');
  $x->withdraw();
  my $sw = $mw->screenwidth();
  my $sh = $mw->screenheight();
  my $title = $x->Photo("title", -file => $filename); 
  $x->overrideredirect(1);
  my $l = $x->Label(-image => 'title', -bd => 1, -relief => 'sunken');
  $l->pack(-side => 'top', -expand => 1, -fill => 'both');
  my $xpos = ($sw - 200)/2;
  my $ypos = ($sh - 250)/2;
  $x->geometry("+$xpos+$ypos");
  $x->deiconify();
  $x->idletasks();
  $mw->after(3000, sub {$x->destroy});
}

sub cbb_setup {
  my($mw)= @_;
  
  # ??? acctSetClean;

  # ???
  # open a two way pipe to the perl engine.
  #set eng [open |$lib_path/wrapper.pl r+];
  
  # Set debugging flag on/off in perl engine
  #puts $eng "debug $cbb(debug)"; flush $eng;
  #gets $eng result;
  #if { $cbb(debug) } { puts $result }
  
  # Set cache flag on/off in perl engine
  #puts $eng "cache $cbb(cache)"; flush $eng;
  #gets $eng result;
  #if { $cbb(debug) } { puts $result }
  
  # If Crypt is activated, ask for password
  if($main::cbb{'use-crypt'}) { useCrypt(); }
  
  # Determine what century we are in (used as the default when not specified)
  #puts $eng "century"; flush $eng;
  #gets $eng cbb(century);
  #if { $cbb(debug) } { puts "Our current year starts with $cbb(century)" }

  # setup the main window
  setup_main($mw);
  
  # make sure we have a backup file being saved ... automatically
  #  setup_auto_save
  
  # initialize main window bindings
    #setup_default_binding
  
  # initialize undo
  #undoInit
  
  # Remind user to remove ~/.cbbrc if it still exists.
  #if { [file exists "~/.cbbrc"] } {
  #cbbWindow.ok "You can remove your ``.cbbrc'' file because it is 
  #	obsolete.  It has been replaced with a ``.cbbrc.tcl'' file."
  #tkwait window .ok
  # }
  
  # If the user doesn't have a ~/.cbbrc.tcl file, create one for him.
  # if { [file exists $cbb(conf_file)] } {
  # it exists, don't bother right now
  # } else {
  #cbbrcSave
  #  }
  
  # This should remain the last thing in the setup procedure.  It leaves 
    # a welcome message at the bottom of the window.
  
  # .status.line configure -text "Welcome to the Check Book Balancer."
  # update
}

# ----------------------------------------------------------------------------
# Parse command line arguments
# ----------------------------------------------------------------------------

sub startup_cbb {
  cbb_splash($mw, $pref_image_dir . '/splash-32.gif');  

  # Load the rest of the tcl pieces
  my $file;
  foreach $file ( @main::source_files ) { require $file; }

  CBBlib::set_debug(1) if $pref_debug;
  CBBlib::set_verbose(1) if $pref_verbose;

  # initialize variables, parse command line, setup main window, etc.
  cbb_setup($main::mw);
}



1;
__END__

# ----------------------------------------------------------------------------
# Initialize CBB
# ----------------------------------------------------------------------------


# ----------------------------------------------------------------------------
# $Log: init.pl,v $
# Revision 1.2  1998/01/24 02:21:25  rlb
# Many changes.  Hopefully I'll be better about commits now.
#
# Revision 1.1  1997/10/10 18:15:54  rlb
# Initial submission
#
# Revision 2.18  1997/06/12 21:53:14  curt
# Applied more patches from Martin Schenk <schenkm@ping.at>.  His changes
# greatly improved and developed the preferences menu so it is now actually
# usable to set preferences.
# Current data file is now saved in ~/.cbbcur.tcl
#
# Revision 2.17  1997/05/06 01:00:24  curt
# Added patches contributed by Martin Schenk <schenkm@ping.at>
# - Default to umask of 066 so .CBB files get created rw by owner only
# - Added support for pgp encrypting data files
# - Added support for displaying only recent parts of files (avoids
#   waiting to load in lots of old txns you don't currently need.)
# - Added a feature to "cache" whole accounts in the perl engine so
#   that switching between accounts can be faster.
# - The above options can be turned on/off via the preferrences menu.
#
# Revision 2.16  1997/04/23 18:08:48  curt
# Source ~/.wishrc.tcl on startup if it exists.
#
# Revision 2.15  1997/04/16 20:30:25  curt
# Reduced color version of splash.gif renamed to splash-32.gif
#
# Revision 2.14  1997/04/07 21:13:33  curt
# Add a $cbb(web_browser) variable which can be set from ~/.cbbrc.tcl
# If netscape is the browser, and there is a copy running, don't start
# a new copy.
#
# Revision 2.13  1997/04/03 03:54:48  curt
# Enable auto save without prompting when switching to a new account.
# Contributed by Jonathan I. Kamens <jik@kamens.brookline.ma.us>
#
# Revision 2.12  1997/04/03 03:40:29  curt
# Added a --version option.
#
# Revision 2.11  1997/01/10 22:03:31  curt
# Transfer fixups and a few other misc. fixes contributed by
#   Lionel Mallet <Lionel.Mallet@sophia.inria.fr>
#
# Revision 2.10  1997/01/09 04:03:03  curt
# Allow user to specify colors of alternating transaction lines.
#
# Revision 2.9  1997/01/09 03:56:58  curt
# Added contrib script loan.pl.
# User sizable account list.
# Removed some old gnuplot baggage from install.pl.
#
# Revision 2.8  1997/01/02 04:38:35  curt
# Changes over the 1996 holidays:
#   - Converted listbox to text widget.  This allows us to do nice
#     things with alternating background colors, highliting, red
#     negative numbers, etc.
#   - Negative transactions are now drawn in red.
#   - Added a Goto <Today> option.
#   - <Home> & <End> were double bound.  Now, listbox can be traversed with
#     <Meta-Home> and <Meta-End>
#
# Revision 2.7  1996/12/17 14:53:58  curt
# Updated copyright date.
#
# Revision 2.6  1996/12/16 04:18:18  curt
# Continuing the great overhaul of December 1996.
#
# Revision 2.5  1996/12/14 17:15:23  curt
# The great overhaul of December '96.
#
# Revision 2.4  1996/12/11 04:32:29  curt
# Several minor tweaks.
#
# Revision 2.3  1996/12/11 01:03:42  curt
# Added balloon help support.
#
# Revision 2.2  1996/12/09 23:54:54  curt
# Updated splash screen image.
#
# Revision 2.1  1996/12/08 07:37:49  curt
# Initial revision.
#

