#!/usr/bin/perl -w

use strict;
use English;
use IO;
use Tk;

my $pref_button_font = '-schumacher-clean-medium-r-normal--12-*';

my $data = [ 'a', 'x', 'h', 'g', 'r'];

sub find_ballpark {
  my($data, $item, $comparator, $left, $right) = @_;
  # Good old fashioned binary search...
  $left = 0 unless $left; $right = 0 unless $right;
  
  $mid = $left + (($right - $left) >> 2);
  if($mid == 
  if(&$comparator($item, $$data[$mid]) == 0) {
    return $mid;
  } else {
    return

}

sub find_insert_position {
  my($data, $item, $comparator) = @_;
 
  my $i = find_ballpark($data, $item, $comparator);
  while($$data[$i + 1] && (&$comparator($i, $$data[$i + 1]) < 0)) {
    $i++;
  }
  return $i;
}
  

sub list_win_insert {
  my($self, $items) = @_;
  my @sorted_items = sort $$self{'user-cmp-function'} @$items;
  my $data_items = $$self{'data-items'};
  
  map { 
    my $insertion_index =
        find_insert_position($data_items, $_, \&compare_data);
    $list->insert($insertion_index, &$data_to_string($_));
    splice @$data_items, $insert_position, 0, $_;
  } @sorted_items;
}

sub add_button {
  my($parent, $name, $help_string, $function) = @_;

  my $button = $parent->Button(-text => $name,
                               -font => $pref_button_font,
                               -takefocus => 0,
                               -command => $function);

  $button->pack(-side => 'left', -expand => 1);
}

sub list_win_create {
  my($parent, $singular_name, $user_functions, $callbacks) = @_;

  # data_from_user_input() queries the user and returns a new instance of $data
  # should return undef or 0 on failure.

  # start_edit_data($data) should query the user for edits.  It is the
  # responsibility of this code to make sure that
  # list_win_note_modifications([$data, ...]); is eventually called if
  # appropriate.

  # data_from_string($data) a string for the display list to represent $data
  # cmp_func() compares sort order of $a and $b (both instances of $data)
  #
  # win is the top level window
  # list is the listbox

  my $list_win = { @$user_functions, @$callbacks };
  $$list_win{'data-items'} = [];

  my $win = $$list_win{'win'} = $parent->Toplevel();
  $win->OnDestroy(sub { list_win_close($list_win); });
  
  my $list = $$list_win{'list'} = $win->Listbox(-selectmode => 'extended');
  $list->pack(-side => 'top', -fill => 'both', -expand => 1);
  
  my $bar = $win->Frame(-relief => 'sunken');
  add_button($bar, 'New', 'Create a new $singular_name.', sub {
    list_win_query_new_item($list_win);
  };    
  add_button($bar, 'Edit', 'Edit selected ${singular_name}s.', sub {
    my @items = $list->curselection();
    list_win_edit_items($list_win, \@items);
  });
  add_button($bar, 'Delete', 'Delete selected ${singular_name}s.', sub {
    my @items = $list->curselection();
    list_win_delete_indices($list_win, \@items);
  });
  add_button($bar, 'Close', 'Close window.',
             sub { list_win_close($list_win); });
  $bar->pack(-side => 'bottom', -fill => 'x');
}

functions:
    create(data_from_user_input(), data_to_string(), cmp_func())
    insert([data, ...])
    my $make_data =  $$list_win{'data-from-user-input'};
    my $data = &$make_data($list_win);
    if($data) {
      list_win_insert([$data]);
    }
    delete(data, ...)
    delete(index, ...)
    map { $list->delete_index($_); } reverse sort @items;
    modify([data, ])

user_provided:
    data_from_user_input()
    edit_data($data)
    data_to_string(data)
    cmp_func()  $a $b

callbacks:
    "insert" (data, ...)
    "delete" (data, ...)
    "modify" (data, ...)

my $mw  = new MainWindow;
  $list->insert('end', @$data);


MainLoop;
